@@ -17,19 +17,19 @@
 
 // written by Pierre Chifflier  <chifflier@wzdftpd.net>
 
-use libc;
 use std;
 use std::ffi::{CStr,CString};
-use nom::{IResult,be_u32};
+use nom;
+use nom::be_u32;
 use der_parser::der_read_element_header;
 use kerberos_parser::krb5_parser;
 use kerberos_parser::krb5::{EncryptionType,ErrorCode,MessageType,PrincipalName,Realm};
-use applayer;
-use core;
-use core::{AppProto,Flow,ALPROTO_FAILED,ALPROTO_UNKNOWN,STREAM_TOCLIENT,STREAM_TOSERVER,sc_detect_engine_state_free};
-use parser::*;
+use crate::applayer;
+use crate::core;
+use crate::core::{AppProto,Flow,ALPROTO_FAILED,ALPROTO_UNKNOWN,STREAM_TOCLIENT,STREAM_TOSERVER,sc_detect_engine_state_free};
+use crate::parser::*;
 
-use log::*;
+use crate::log::*;
 
 #[repr(u32)]
 pub enum KRB5Event {
@@ -37,6 +37,16 @@ pub enum KRB5Event {
     WeakEncryption,
 }
 
+impl KRB5Event {
+    fn from_i32(value: i32) -> Option<KRB5Event> {
+        match value {
+            0 => Some(KRB5Event::MalformedData),
+            1 => Some(KRB5Event::WeakEncryption),
+            _ => None,
+        }
+    }
+}
+
 pub struct KRB5State {
     pub req_id: u8,
 
@@ -79,6 +89,7 @@ pub struct KRB5Transaction {
     events: *mut core::AppLayerDecoderEvents,
 
     logged: applayer::LoggerFlags,
+    detect_flags: applayer::TxDetectFlags,
 }
 
 pub fn to_hex_string(bytes: &[u8]) -> String {
@@ -107,7 +118,7 @@ impl KRB5State {
     /// Returns The number of messages parsed, or -1 on error
     fn parse(&mut self, i: &[u8], _direction: u8) -> i32 {
         match der_read_element_header(i) {
-            IResult::Done(_rem,hdr) => {
+            Ok((_rem,hdr)) => {
                 // Kerberos messages start with an APPLICATION header
                 if hdr.class != 0b01 { return 1; }
                 match hdr.tag {
@@ -116,7 +127,7 @@ impl KRB5State {
                     },
                     11 => {
                         let res = krb5_parser::parse_as_rep(i);
-                        res.map(|kdc_rep| {
+                        if let Ok((_,kdc_rep)) = res {
                             let mut tx = self.new_tx();
                             tx.msg_type = MessageType::KRB_AS_REP;
                             tx.cname = Some(kdc_rep.cname);
@@ -127,7 +138,7 @@ impl KRB5State {
                             if test_weak_encryption(kdc_rep.enc_part.etype) {
                                 self.set_event(KRB5Event::WeakEncryption);
                             }
-                        });
+                        };
                         self.req_id = 0;
                     },
                     12 => {
@@ -135,7 +146,7 @@ impl KRB5State {
                     },
                     13 => {
                         let res = krb5_parser::parse_tgs_rep(i);
-                        res.map(|kdc_rep| {
+                        if let Ok((_,kdc_rep)) = res {
                             let mut tx = self.new_tx();
                             tx.msg_type = MessageType::KRB_TGS_REP;
                             tx.cname = Some(kdc_rep.cname);
@@ -146,7 +157,7 @@ impl KRB5State {
                             if test_weak_encryption(kdc_rep.enc_part.etype) {
                                 self.set_event(KRB5Event::WeakEncryption);
                             }
-                        });
+                        };
                         self.req_id = 0;
                     },
                     14 => {
@@ -157,7 +168,7 @@ impl KRB5State {
                     },
                     30 => {
                         let res = krb5_parser::parse_krb_error(i);
-                        res.map(|error| {
+                        if let Ok((_,error)) = res {
                             let mut tx = self.new_tx();
                             tx.msg_type = MessageType(self.req_id as u32);
                             tx.cname = error.cname;
@@ -165,19 +176,19 @@ impl KRB5State {
                             tx.sname = Some(error.sname);
                             tx.error_code = Some(error.error_code);
                             self.transactions.push(tx);
-                        });
+                        };
                         self.req_id = 0;
                     },
                     _ => { SCLogDebug!("unknown/unsupported tag {}", hdr.tag); },
                 }
                 0
             },
-            IResult::Incomplete(_) => {
+            Err(nom::Err::Incomplete(_)) => {
                 SCLogDebug!("Insufficient data while parsing KRB5 data");
                 self.set_event(KRB5Event::MalformedData);
                 -1
             },
-            IResult::Error(_) => {
+            Err(_) => {
                 SCLogDebug!("Error while parsing KRB5 data");
                 self.set_event(KRB5Event::MalformedData);
                 -1
@@ -230,6 +241,7 @@ impl KRB5Transaction {
             de_state: None,
             events: std::ptr::null_mut(),
             logged: applayer::LoggerFlags::new(),
+            detect_flags: applayer::TxDetectFlags::default(),
         }
     }
 }
@@ -264,7 +276,7 @@ pub fn test_weak_encryption(alg:EncryptionType) -> bool {
 
 /// Returns *mut KRB5State
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_new() -> *mut libc::c_void {
+pub extern "C" fn rs_krb5_state_new() -> *mut std::os::raw::c_void {
     let state = KRB5State::new();
     let boxed = Box::new(state);
     return unsafe{std::mem::transmute(boxed)};
@@ -273,16 +285,16 @@ pub extern "C" fn rs_krb5_state_new() -> *mut libc::c_void {
 /// Params:
 /// - state: *mut KRB5State as void pointer
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_free(state: *mut libc::c_void) {
+pub extern "C" fn rs_krb5_state_free(state: *mut std::os::raw::c_void) {
     // Just unbox...
     let mut state: Box<KRB5State> = unsafe{std::mem::transmute(state)};
     state.free();
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_get_tx(state: *mut libc::c_void,
+pub extern "C" fn rs_krb5_state_get_tx(state: *mut std::os::raw::c_void,
                                       tx_id: u64)
-                                      -> *mut libc::c_void
+                                      -> *mut std::os::raw::c_void
 {
     let state = cast_pointer!(state,KRB5State);
     match state.get_tx_by_id(tx_id) {
@@ -292,7 +304,7 @@ pub extern "C" fn rs_krb5_state_get_tx(state: *mut libc::c_void,
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_get_tx_count(state: *mut libc::c_void)
+pub extern "C" fn rs_krb5_state_get_tx_count(state: *mut std::os::raw::c_void)
                                             -> u64
 {
     let state = cast_pointer!(state,KRB5State);
@@ -300,7 +312,7 @@ pub extern "C" fn rs_krb5_state_get_tx_count(state: *mut libc::c_void)
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_tx_free(state: *mut libc::c_void,
+pub extern "C" fn rs_krb5_state_tx_free(state: *mut std::os::raw::c_void,
                                        tx_id: u64)
 {
     let state = cast_pointer!(state,KRB5State);
@@ -310,22 +322,22 @@ pub extern "C" fn rs_krb5_state_tx_free(state: *mut libc::c_void,
 #[no_mangle]
 pub extern "C" fn rs_krb5_state_progress_completion_status(
     _direction: u8)
-    -> libc::c_int
+    -> std::os::raw::c_int
 {
     return 1;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_tx_get_alstate_progress(_tx: *mut libc::c_void,
+pub extern "C" fn rs_krb5_tx_get_alstate_progress(_tx: *mut std::os::raw::c_void,
                                                  _direction: u8)
-                                                 -> libc::c_int
+                                                 -> std::os::raw::c_int
 {
     1
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_tx_set_logged(_state: *mut libc::c_void,
-                                       tx: *mut libc::c_void,
+pub extern "C" fn rs_krb5_tx_set_logged(_state: *mut std::os::raw::c_void,
+                                       tx: *mut std::os::raw::c_void,
                                        logged: u32)
 {
     let tx = cast_pointer!(tx,KRB5Transaction);
@@ -333,8 +345,8 @@ pub extern "C" fn rs_krb5_tx_set_logged(_state: *mut libc::c_void,
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_tx_get_logged(_state: *mut libc::c_void,
-                                       tx: *mut libc::c_void)
+pub extern "C" fn rs_krb5_tx_get_logged(_state: *mut std::os::raw::c_void,
+                                       tx: *mut std::os::raw::c_void)
                                        -> u32
 {
     let tx = cast_pointer!(tx,KRB5Transaction);
@@ -344,8 +356,8 @@ pub extern "C" fn rs_krb5_tx_get_logged(_state: *mut libc::c_void,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_state_set_tx_detect_state(
-    tx: *mut libc::c_void,
-    de_state: &mut core::DetectEngineState) -> libc::c_int
+    tx: *mut std::os::raw::c_void,
+    de_state: &mut core::DetectEngineState) -> std::os::raw::c_int
 {
     let tx = cast_pointer!(tx,KRB5Transaction);
     tx.de_state = Some(de_state);
@@ -354,7 +366,7 @@ pub extern "C" fn rs_krb5_state_set_tx_detect_state(
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_state_get_tx_detect_state(
-    tx: *mut libc::c_void)
+    tx: *mut std::os::raw::c_void)
     -> *mut core::DetectEngineState
 {
     let tx = cast_pointer!(tx,KRB5Transaction);
@@ -364,24 +376,40 @@ pub extern "C" fn rs_krb5_state_get_tx_detect_state(
     }
 }
 
-
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_get_events(state: *mut libc::c_void,
-                                          tx_id: u64)
-                                          -> *mut core::AppLayerDecoderEvents
+pub extern "C" fn rs_krb5_state_get_event_info_by_id(event_id: std::os::raw::c_int,
+                                                     event_name: *mut *const std::os::raw::c_char,
+                                                     event_type: *mut core::AppLayerEventType)
+                                                     -> i8
 {
-    let state = cast_pointer!(state,KRB5State);
-    match state.get_tx_by_id(tx_id) {
-        Some(tx) => tx.events,
-        _        => std::ptr::null_mut(),
+    if let Some(e) = KRB5Event::from_i32(event_id as i32) {
+        let estr = match e {
+            KRB5Event::MalformedData  => { "malformed_data\0" },
+            KRB5Event::WeakEncryption => { "weak_encryption\0" },
+        };
+        unsafe{
+            *event_name = estr.as_ptr() as *const std::os::raw::c_char;
+            *event_type = core::APP_LAYER_EVENT_TYPE_TRANSACTION;
+        };
+        0
+    } else {
+        -1
     }
 }
 
 #[no_mangle]
-pub extern "C" fn rs_krb5_state_get_event_info(event_name: *const libc::c_char,
-                                              event_id: *mut libc::c_int,
+pub extern "C" fn rs_krb5_state_get_events(tx: *mut std::os::raw::c_void)
+                                          -> *mut core::AppLayerDecoderEvents
+{
+    let tx = cast_pointer!(tx, KRB5Transaction);
+    return tx.events;
+}
+
+#[no_mangle]
+pub extern "C" fn rs_krb5_state_get_event_info(event_name: *const std::os::raw::c_char,
+                                              event_id: *mut std::os::raw::c_int,
                                               event_type: *mut core::AppLayerEventType)
-                                              -> libc::c_int
+                                              -> std::os::raw::c_int
 {
     if event_name == std::ptr::null() { return -1; }
     let c_event_name: &CStr = unsafe { CStr::from_ptr(event_name) };
@@ -397,7 +425,7 @@ pub extern "C" fn rs_krb5_state_get_event_info(event_name: *const libc::c_char,
     };
     unsafe{
         *event_type = core::APP_LAYER_EVENT_TYPE_TRANSACTION;
-        *event_id = event as libc::c_int;
+        *event_id = event as std::os::raw::c_int;
     };
     0
 }
@@ -405,12 +433,15 @@ static mut ALPROTO_KRB5 : AppProto = ALPROTO_UNKNOWN;
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_probing_parser(_flow: *const Flow,
-        input:*const u8, input_len: u32) -> AppProto {
+        _direction: u8,
+        input:*const u8, input_len: u32,
+        _rdir: *mut u8) -> AppProto
+{
     let slice = build_slice!(input,input_len as usize);
     let alproto = unsafe{ ALPROTO_KRB5 };
     if slice.len() <= 10 { return unsafe{ALPROTO_FAILED}; }
     match der_read_element_header(slice) {
-        IResult::Done(rem, ref hdr) => {
+        Ok((rem, ref hdr)) => {
             // Kerberos messages start with an APPLICATION header
             if hdr.class != 0b01 { return unsafe{ALPROTO_FAILED}; }
             // Tag number should be <= 30
@@ -418,7 +449,7 @@ pub extern "C" fn rs_krb5_probing_parser(_flow: *const Flow,
             // Kerberos messages contain sequences
             if rem.is_empty() || rem[0] != 0x30 { return unsafe{ALPROTO_FAILED}; }
             // Check kerberos version
-            if let IResult::Done(rem,_hdr) = der_read_element_header(rem) {
+            if let Ok((rem,_hdr)) = der_read_element_header(rem) {
                 if rem.len() > 5 {
                     match (rem[2],rem[3],rem[4]) {
                         // Encoding of DER integer 5 (version)
@@ -429,10 +460,10 @@ pub extern "C" fn rs_krb5_probing_parser(_flow: *const Flow,
             }
             return unsafe{ALPROTO_FAILED};
         },
-        IResult::Incomplete(_) => {
+        Err(nom::Err::Incomplete(_)) => {
             return ALPROTO_UNKNOWN;
         },
-        IResult::Error(_) => {
+        Err(_) => {
             return unsafe{ALPROTO_FAILED};
         },
     }
@@ -440,19 +471,23 @@ pub extern "C" fn rs_krb5_probing_parser(_flow: *const Flow,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_probing_parser_tcp(_flow: *const Flow,
-        input:*const u8, input_len: u32) -> AppProto {
+        direction: u8,
+        input:*const u8, input_len: u32,
+        rdir: *mut u8) -> AppProto
+{
     let slice = build_slice!(input,input_len as usize);
     if slice.len() <= 14 { return unsafe{ALPROTO_FAILED}; }
     match be_u32(slice) {
-        IResult::Done(rem, record_mark) => {
+        Ok((rem, record_mark)) => {
             // protocol implementations forbid very large requests
             if record_mark > 16384 { return unsafe{ALPROTO_FAILED}; }
-            return rs_krb5_probing_parser(_flow, rem.as_ptr(), rem.len() as u32);
+            return rs_krb5_probing_parser(_flow, direction,
+                    rem.as_ptr(), rem.len() as u32, rdir);
         },
-        IResult::Incomplete(_) => {
+        Err(nom::Err::Incomplete(_)) => {
             return ALPROTO_UNKNOWN;
         },
-        IResult::Error(_) => {
+        Err(_) => {
             return unsafe{ALPROTO_FAILED};
         },
     }
@@ -460,11 +495,11 @@ pub extern "C" fn rs_krb5_probing_parser_tcp(_flow: *const Flow,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_parse_request(_flow: *const core::Flow,
-                                       state: *mut libc::c_void,
-                                       _pstate: *mut libc::c_void,
+                                       state: *mut std::os::raw::c_void,
+                                       _pstate: *mut std::os::raw::c_void,
                                        input: *const u8,
                                        input_len: u32,
-                                       _data: *const libc::c_void,
+                                       _data: *const std::os::raw::c_void,
                                        _flags: u8) -> i32 {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
@@ -473,11 +508,11 @@ pub extern "C" fn rs_krb5_parse_request(_flow: *const core::Flow,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_parse_response(_flow: *const core::Flow,
-                                       state: *mut libc::c_void,
-                                       _pstate: *mut libc::c_void,
+                                       state: *mut std::os::raw::c_void,
+                                       _pstate: *mut std::os::raw::c_void,
                                        input: *const u8,
                                        input_len: u32,
-                                       _data: *const libc::c_void,
+                                       _data: *const std::os::raw::c_void,
                                        _flags: u8) -> i32 {
     let buf = build_slice!(input,input_len as usize);
     let state = cast_pointer!(state,KRB5State);
@@ -486,11 +521,11 @@ pub extern "C" fn rs_krb5_parse_response(_flow: *const core::Flow,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
-                                       state: *mut libc::c_void,
-                                       _pstate: *mut libc::c_void,
+                                       state: *mut std::os::raw::c_void,
+                                       _pstate: *mut std::os::raw::c_void,
                                        input: *const u8,
                                        input_len: u32,
-                                       _data: *const libc::c_void,
+                                       _data: *const std::os::raw::c_void,
                                        _flags: u8) -> i32 {
     if input_len < 4 { return -1; }
     let buf = build_slice!(input,input_len as usize);
@@ -503,7 +538,7 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
         _ => {
             // sanity check to avoid memory exhaustion
             if state.defrag_buf_ts.len() + buf.len() > 100000 {
-                SCLogDebug!("rs_krb5_parse_resquest_tcp: TCP buffer exploded {} {}",
+                SCLogDebug!("rs_krb5_parse_request_tcp: TCP buffer exploded {} {}",
                             state.defrag_buf_ts.len(), buf.len());
                 return 1;
             }
@@ -516,7 +551,7 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
     while cur_i.len() > 0 {
         if state.record_ts == 0 {
             match be_u32(cur_i) {
-                IResult::Done(rem,record) => {
+                Ok((rem,record)) => {
                     state.record_ts = record as usize;
                     cur_i = rem;
                 },
@@ -544,11 +579,11 @@ pub extern "C" fn rs_krb5_parse_request_tcp(_flow: *const core::Flow,
 
 #[no_mangle]
 pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
-                                       state: *mut libc::c_void,
-                                       _pstate: *mut libc::c_void,
+                                       state: *mut std::os::raw::c_void,
+                                       _pstate: *mut std::os::raw::c_void,
                                        input: *const u8,
                                        input_len: u32,
-                                       _data: *const libc::c_void,
+                                       _data: *const std::os::raw::c_void,
                                        _flags: u8) -> i32 {
     if input_len < 4 { return -1; }
     let buf = build_slice!(input,input_len as usize);
@@ -574,7 +609,7 @@ pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
     while cur_i.len() > 0 {
         if state.record_tc == 0 {
             match be_u32(cur_i) {
-                IResult::Done(rem,record) => {
+                Ok((rem,record)) => {
                     state.record_tc = record as usize;
                     cur_i = rem;
                 },
@@ -600,6 +635,8 @@ pub extern "C" fn rs_krb5_parse_response_tcp(_flow: *const core::Flow,
     status
 }
 
+export_tx_detect_flags_set!(rs_krb5_tx_detect_flags_set, KRB5Transaction);
+export_tx_detect_flags_get!(rs_krb5_tx_detect_flags_get, KRB5Transaction);
 
 const PARSER_NAME : &'static [u8] = b"krb5\0";
 
@@ -607,34 +644,37 @@ const PARSER_NAME : &'static [u8] = b"krb5\0";
 pub unsafe extern "C" fn rs_register_krb5_parser() {
     let default_port = CString::new("88").unwrap();
     let mut parser = RustParser {
-        name              : PARSER_NAME.as_ptr() as *const libc::c_char,
-        default_port      : default_port.as_ptr(),
-        ipproto           : libc::IPPROTO_UDP,
-        probe_ts          : rs_krb5_probing_parser,
-        probe_tc          : rs_krb5_probing_parser,
-        min_depth         : 0,
-        max_depth         : 16,
-        state_new         : rs_krb5_state_new,
-        state_free        : rs_krb5_state_free,
-        tx_free           : rs_krb5_state_tx_free,
-        parse_ts          : rs_krb5_parse_request,
-        parse_tc          : rs_krb5_parse_response,
-        get_tx_count      : rs_krb5_state_get_tx_count,
-        get_tx            : rs_krb5_state_get_tx,
-        tx_get_comp_st    : rs_krb5_state_progress_completion_status,
-        tx_get_progress   : rs_krb5_tx_get_alstate_progress,
-        get_tx_logged     : Some(rs_krb5_tx_get_logged),
-        set_tx_logged     : Some(rs_krb5_tx_set_logged),
-        get_de_state      : rs_krb5_state_get_tx_detect_state,
-        set_de_state      : rs_krb5_state_set_tx_detect_state,
-        get_events        : Some(rs_krb5_state_get_events),
-        get_eventinfo     : Some(rs_krb5_state_get_event_info),
-        localstorage_new  : None,
-        localstorage_free : None,
-        get_tx_mpm_id     : None,
-        set_tx_mpm_id     : None,
-        get_files         : None,
-        get_tx_iterator   : None,
+        name               : PARSER_NAME.as_ptr() as *const std::os::raw::c_char,
+        default_port       : default_port.as_ptr(),
+        ipproto            : core::IPPROTO_UDP,
+        probe_ts           : rs_krb5_probing_parser,
+        probe_tc           : rs_krb5_probing_parser,
+        min_depth          : 0,
+        max_depth          : 16,
+        state_new          : rs_krb5_state_new,
+        state_free         : rs_krb5_state_free,
+        tx_free            : rs_krb5_state_tx_free,
+        parse_ts           : rs_krb5_parse_request,
+        parse_tc           : rs_krb5_parse_response,
+        get_tx_count       : rs_krb5_state_get_tx_count,
+        get_tx             : rs_krb5_state_get_tx,
+        tx_get_comp_st     : rs_krb5_state_progress_completion_status,
+        tx_get_progress    : rs_krb5_tx_get_alstate_progress,
+        get_tx_logged      : Some(rs_krb5_tx_get_logged),
+        set_tx_logged      : Some(rs_krb5_tx_set_logged),
+        get_de_state       : rs_krb5_state_get_tx_detect_state,
+        set_de_state       : rs_krb5_state_set_tx_detect_state,
+        get_events         : Some(rs_krb5_state_get_events),
+        get_eventinfo      : Some(rs_krb5_state_get_event_info),
+        get_eventinfo_byid : Some(rs_krb5_state_get_event_info_by_id),
+        localstorage_new   : None,
+        localstorage_free  : None,
+        get_tx_mpm_id      : None,
+        set_tx_mpm_id      : None,
+        get_files          : None,
+        get_tx_iterator    : None,
+        get_tx_detect_flags: Some(rs_krb5_tx_detect_flags_get),
+        set_tx_detect_flags: Some(rs_krb5_tx_detect_flags_set),
     };
     // register UDP parser
     let ip_proto_str = CString::new("udp").unwrap();
@@ -646,10 +686,10 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
             let _ = AppLayerRegisterParser(&parser, alproto);
         }
     } else {
-        SCLogDebug!("Protocol detecter and parser disabled for KRB5/UDP.");
+        SCLogDebug!("Protocol detector and parser disabled for KRB5/UDP.");
     }
     // register TCP parser
-    parser.ipproto = libc::IPPROTO_TCP;
+    parser.ipproto = core::IPPROTO_TCP;
     parser.probe_ts = rs_krb5_probing_parser_tcp;
     parser.probe_tc = rs_krb5_probing_parser_tcp;
     parser.parse_ts = rs_krb5_parse_request_tcp;
@@ -663,6 +703,6 @@ pub unsafe extern "C" fn rs_register_krb5_parser() {
             let _ = AppLayerRegisterParser(&parser, alproto);
         }
     } else {
-        SCLogDebug!("Protocol detecter and parser disabled for KRB5/TCP.");
+        SCLogDebug!("Protocol detector and parser disabled for KRB5/TCP.");
     }
 }
