@@ -15,18 +15,24 @@
  * 02110-1301, USA.
  */
 
-#![allow(ellipsis_inclusive_range_patterns)] // TODO: Remove when MSRV is higher than 1.24
 #![cfg_attr(feature = "strict", deny(warnings))]
 
-extern crate libc;
-
 #[macro_use]
 extern crate nom;
 
+#[macro_use]
+extern crate bitflags;
+extern crate byteorder;
 extern crate crc;
+extern crate memchr;
+#[macro_use]
+extern crate num_derive;
+extern crate widestring;
 
 extern crate der_parser;
 extern crate kerberos_parser;
+extern crate tls_parser;
+extern crate x509_parser;
 
 #[macro_use]
 pub mod log;
@@ -54,8 +60,11 @@ pub mod smb;
 pub mod krb;
 
 pub mod ikev2;
+pub mod snmp;
 
 pub mod ntp;
 pub mod tftp;
 pub mod dhcp;
+pub mod sip;
 pub mod applayertemplate;
+pub mod rdp;
