@@ -18,26 +18,25 @@
 // written by Victor Julien
 // TCP buffering code written by Pierre Chifflier
 
-extern crate libc;
 use std;
 use std::mem::transmute;
 use std::collections::{HashMap};
 use std::ffi::CStr;
 
-use nom::IResult;
+use nom;
 
-use log::*;
-use applayer;
-use applayer::LoggerFlags;
-use core::*;
-use filetracker::*;
-use filecontainer::*;
+use crate::log::*;
+use crate::applayer;
+use crate::applayer::LoggerFlags;
+use crate::core::*;
+use crate::filetracker::*;
+use crate::filecontainer::*;
 
-use nfs::types::*;
-use nfs::rpc_records::*;
-use nfs::nfs_records::*;
-use nfs::nfs2_records::*;
-use nfs::nfs3_records::*;
+use crate::nfs::types::*;
+use crate::nfs::rpc_records::*;
+use crate::nfs::nfs_records::*;
+use crate::nfs::nfs2_records::*;
+use crate::nfs::nfs3_records::*;
 
 pub static mut SURICATA_NFS_FILE_CONFIG: Option<&'static SuricataFileContext> = None;
 
@@ -91,6 +90,18 @@ pub enum NFSEvent {
     UnsupportedVersion = 2,
 }
 
+impl NFSEvent {
+    fn from_i32(value: i32) -> Option<NFSEvent> {
+        match value {
+            0 => Some(NFSEvent::MalformedData),
+            1 => Some(NFSEvent::NonExistingVersion),
+            2 => Some(NFSEvent::UnsupportedVersion),
+            _ => None,
+        }
+    }
+}
+
+
 #[derive(Debug)]
 pub enum NFSTransactionTypeData {
     RENAME(Vec<u8>),
@@ -108,6 +119,10 @@ pub struct NFSTransactionFile {
     /// last xid of this file transfer. Last READ or COMMIT normally.
     pub file_last_xid: u32,
 
+    /// after a gap, this will be set to a time in the future. If the file
+    /// receives no updates before that, it will be considered complete.
+    pub post_gap_ts: u64,
+
     /// file tracker for a single file. Boxed so that we don't use
     /// as much space if we're not a file tx.
     pub file_tracker: FileTransferTracker,
@@ -119,6 +134,7 @@ impl NFSTransactionFile {
             file_additional_procs: Vec::new(),
             chunk_count:0,
             file_last_xid: 0,
+            post_gap_ts: 0,
             file_tracker: FileTransferTracker::new(),
         }
     }
@@ -327,12 +343,20 @@ pub struct NFSState {
 
     is_udp: bool,
 
+    /// true as long as we have file txs that are in a post-gap
+    /// state. It means we'll do extra house keeping for those.
+    check_post_gap_file_txs: bool,
+
     pub nfs_version: u16,
 
     pub events: u16,
 
     /// tx counter for assigning incrementing id's to tx's
     tx_id: u64,
+
+    /// Timestamp in seconds of last update. This is packet time,
+    /// potentially coming from pcaps.
+    ts: u64,
 }
 
 impl NFSState {
@@ -355,9 +379,11 @@ impl NFSState {
             ts_gap:false,
             tc_gap:false,
             is_udp:false,
+            check_post_gap_file_txs:false,
             nfs_version:0,
             events:0,
             tx_id:0,
+            ts: 0,
         }
     }
     pub fn free(&mut self) {
@@ -472,9 +498,74 @@ impl NFSState {
         }
     }
 
+    fn post_gap_housekeeping_for_files(&mut self)
+    {
+        let mut post_gap_txs = false;
+        for tx in &mut self.transactions {
+            if let Some(NFSTransactionTypeData::FILE(ref f)) = tx.type_data {
+                if f.post_gap_ts > 0 {
+                    if self.ts > f.post_gap_ts {
+                        tx.request_done = true;
+                        tx.response_done = true;
+                    } else {
+                        post_gap_txs = true;
+                    }
+                }
+            }
+        }
+        self.check_post_gap_file_txs = post_gap_txs;
+    }
+
+    /* after a gap we will consider all transactions complete for our
+     * direction. File transfer transactions are an exception. Those
+     * can handle gaps. For the file transactions we set the current
+     * (flow) time and prune them in 60 seconds if no update for them
+     * was received. */
+    fn post_gap_housekeeping(&mut self, dir: u8)
+    {
+        if self.ts_ssn_gap && dir == STREAM_TOSERVER {
+            for tx in &mut self.transactions {
+                if tx.id >= self.tx_id {
+                    SCLogDebug!("post_gap_housekeeping: done");
+                    break;
+                }
+                if let Some(NFSTransactionTypeData::FILE(ref mut f)) = tx.type_data {
+                    // leaving FILE txs open as they can deal with gaps. We
+                    // remove them after 60 seconds of no activity though.
+                    if f.post_gap_ts == 0 {
+                        f.post_gap_ts = self.ts + 60;
+                        self.check_post_gap_file_txs = true;
+                    }
+                } else {
+                    SCLogDebug!("post_gap_housekeeping: tx {} marked as done TS", tx.id);
+                    tx.request_done = true;
+                }
+            }
+        } else if self.tc_ssn_gap && dir == STREAM_TOCLIENT {
+            for tx in &mut self.transactions {
+                if tx.id >= self.tx_id {
+                    SCLogDebug!("post_gap_housekeeping: done");
+                    break;
+                }
+                if let Some(NFSTransactionTypeData::FILE(ref mut f)) = tx.type_data {
+                    // leaving FILE txs open as they can deal with gaps. We
+                    // remove them after 60 seconds of no activity though.
+                    if f.post_gap_ts == 0 {
+                        f.post_gap_ts = self.ts + 60;
+                        self.check_post_gap_file_txs = true;
+                    }
+                } else {
+                    SCLogDebug!("post_gap_housekeeping: tx {} marked as done TC", tx.id);
+                    tx.request_done = true;
+                    tx.response_done = true;
+                }
+            }
+        }
+    }
+
     pub fn process_request_record_lookup<'b>(&mut self, r: &RpcPacket<'b>, xidmap: &mut NFSRequestXidMap) {
         match parse_nfs3_request_lookup(r.prog_data) {
-            IResult::Done(_, lookup) => {
+            Ok((_, lookup)) => {
                 SCLogDebug!("LOOKUP {:?}", lookup);
                 xidmap.file_name = lookup.name_vec;
             },
@@ -753,6 +844,11 @@ impl NFSState {
                         }
                     }
 
+                    // reset timestamp if we get called after a gap
+                    if tdf.post_gap_ts > 0 {
+                        tdf.post_gap_ts = 0;
+                    }
+
                     tdf.chunk_count += 1;
                     let cs = tdf.file_tracker.update(files, flags, data, gap_size);
                     /* see if we need to close the tx */
@@ -816,7 +912,7 @@ impl NFSState {
 
         if nfs_version == 2 {
             let size = match parse_nfs2_attribs(reply.attr_blob) {
-                IResult::Done(_, ref attr) => {
+                Ok((_, ref attr)) => {
                     attr.asize
                 },
                 _ => { 0 },
@@ -1014,7 +1110,7 @@ impl NFSState {
 
         while cur_i.len() > 0 { // min record size
             match parse_rpc_request_partial(cur_i) {
-                IResult::Done(_, ref rpc_phdr) => {
+                Ok((_, ref rpc_phdr)) => {
                     let rec_size = (rpc_phdr.hdr.frag_len + 4) as usize;
                     //SCLogDebug!("rec_size {}/{}", rec_size, cur_i.len());
                     //SCLogDebug!("cur_i {:?}", cur_i);
@@ -1032,9 +1128,9 @@ impl NFSState {
 
                                 // lets try to parse the RPC record. Might fail with Incomplete.
                                 match parse_rpc(cur_i) {
-                                    IResult::Done(remaining, ref rpc_record) => {
+                                    Ok((remaining, ref rpc_record)) => {
                                         match parse_nfs3_request_write(rpc_record.prog_data) {
-                                            IResult::Done(_, ref nfs_request_write) => {
+                                            Ok((_, ref nfs_request_write)) => {
                                                 // deal with the partial nfs write data
                                                 status |= self.process_partial_write_request_record(rpc_record, nfs_request_write);
                                                 cur_i = remaining; // progress input past parsed record
@@ -1044,14 +1140,15 @@ impl NFSState {
                                             },
                                         }
                                     },
-                                    IResult::Incomplete(_) => {
+                                    Err(nom::Err::Incomplete(_)) => {
                                         // we just size checked for the minimal record size above,
                                         // so if options are used (creds/verifier), we can still
                                         // have Incomplete data. Fall through to the buffer code
                                         // and try again on our next iteration.
                                         SCLogDebug!("TS data incomplete");
                                     },
-                                    IResult::Error(_e) => {
+                                    Err(nom::Err::Error(_e)) |
+                                    Err(nom::Err::Failure(_e)) => {
                                         self.set_event(NFSEvent::MalformedData);
                                         SCLogDebug!("Parsing failed: {:?}", _e);
                                         return 1;
@@ -1066,11 +1163,11 @@ impl NFSState {
                     // we have the full records size worth of data,
                     // let's parse it
                     match parse_rpc(&cur_i[..rec_size]) {
-                        IResult::Done(_, ref rpc_record) => {
+                        Ok((_, ref rpc_record)) => {
                             cur_i = &cur_i[rec_size..];
                             status |= self.process_request_record(rpc_record);
                         },
-                        IResult::Incomplete(_) => {
+                        Err(nom::Err::Incomplete(_)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
 
                             // we shouldn't get incomplete as we have the full data
@@ -1080,25 +1177,33 @@ impl NFSState {
 
                             status = 1;
                         },
-                        IResult::Error(_e) => {
+                        Err(nom::Err::Error(_e)) |
+                        Err(nom::Err::Failure(_e)) => {
                             self.set_event(NFSEvent::MalformedData);
                             SCLogDebug!("Parsing failed: {:?}", _e);
                             return 1;
                         },
                     }
                 },
-                IResult::Incomplete(_) => {
+                Err(nom::Err::Incomplete(_)) => {
                     SCLogDebug!("Fragmentation required (TCP level) 2");
                     self.tcp_buffer_ts.extend_from_slice(cur_i);
                     break;
                 },
-                IResult::Error(_e) => {
+                Err(nom::Err::Error(_e)) |
+                Err(nom::Err::Failure(_e)) => {
                     self.set_event(NFSEvent::MalformedData);
                     SCLogDebug!("Parsing failed: {:?}", _e);
                     return 1;
                 },
             }
         };
+
+        self.post_gap_housekeeping(STREAM_TOSERVER);
+        if self.check_post_gap_file_txs {
+            self.post_gap_housekeeping_for_files();
+        }
+
         status
     }
 
@@ -1168,7 +1273,7 @@ impl NFSState {
 
         while cur_i.len() > 0 {
             match parse_rpc_packet_header(cur_i) {
-                IResult::Done(_, ref rpc_hdr) => {
+                Ok((_, ref rpc_hdr)) => {
                     let rec_size = (rpc_hdr.frag_len + 4) as usize;
                     // see if we have all data available
                     if rec_size > cur_i.len() {
@@ -1184,29 +1289,31 @@ impl NFSState {
 
                                 // we should have enough data to parse the RPC record
                                 match parse_rpc_reply(cur_i) {
-                                    IResult::Done(remaining, ref rpc_record) => {
+                                    Ok((remaining, ref rpc_record)) => {
                                         match parse_nfs3_reply_read(rpc_record.prog_data) {
-                                            IResult::Done(_, ref nfs_reply_read) => {
+                                            Ok((_, ref nfs_reply_read)) => {
                                                 // deal with the partial nfs read data
                                                 status |= self.process_partial_read_reply_record(rpc_record, nfs_reply_read);
                                                 cur_i = remaining; // progress input past parsed record
                                             },
-                                            IResult::Incomplete(_) => {
+                                            Err(nom::Err::Incomplete(_)) => {
                                                 self.set_event(NFSEvent::MalformedData);
                                             },
-                                            IResult::Error(_e) => {
+                                            Err(nom::Err::Error(_e)) |
+                                            Err(nom::Err::Failure(_e)) => {
                                                 self.set_event(NFSEvent::MalformedData);
                                                 SCLogDebug!("Parsing failed: {:?}", _e);
                                                 return 1;
                                             }
                                         }
                                     },
-                                    IResult::Incomplete(_) => {
+                                    Err(nom::Err::Incomplete(_)) => {
                                         // size check was done for MINIMAL record size,
                                         // so Incomplete is normal.
                                         SCLogDebug!("TC data incomplete");
                                     },
-                                    IResult::Error(_e) => {
+                                    Err(nom::Err::Error(_e)) |
+                                    Err(nom::Err::Failure(_e)) => {
                                         self.set_event(NFSEvent::MalformedData);
                                         SCLogDebug!("Parsing failed: {:?}", _e);
                                         return 1;
@@ -1220,11 +1327,11 @@ impl NFSState {
 
                     // we have the full data of the record, lets parse
                     match parse_rpc_reply(&cur_i[..rec_size]) {
-                        IResult::Done(_, ref rpc_record) => {
+                        Ok((_, ref rpc_record)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
                             status |= self.process_reply_record(rpc_record);
                         },
-                        IResult::Incomplete(_) => {
+                        Err(nom::Err::Incomplete(_)) => {
                             cur_i = &cur_i[rec_size..]; // progress input past parsed record
 
                             // we shouldn't get incomplete as we have the full data
@@ -1234,25 +1341,31 @@ impl NFSState {
 
                             status = 1;
                         },
-                        IResult::Error(_e) => {
+                        Err(nom::Err::Error(_e)) |
+                        Err(nom::Err::Failure(_e)) => {
                             self.set_event(NFSEvent::MalformedData);
                             SCLogDebug!("Parsing failed: {:?}", _e);
                             return 1;
                         }
                     }
                 },
-                IResult::Incomplete(_) => {
+                Err(nom::Err::Incomplete(_)) => {
                     SCLogDebug!("REPLY: insufficient data for HDR");
                     self.tcp_buffer_tc.extend_from_slice(cur_i);
                     break;
                 },
-                IResult::Error(_e) => {
+                Err(nom::Err::Error(_e)) |
+                Err(nom::Err::Failure(_e)) => {
                     self.set_event(NFSEvent::MalformedData);
                     SCLogDebug!("Parsing failed: {:?}", _e);
                     return 1;
                 },
             }
         };
+        self.post_gap_housekeeping(STREAM_TOCLIENT);
+        if self.check_post_gap_file_txs {
+            self.post_gap_housekeeping_for_files();
+        }
         status
     }
     /// Parsing function
@@ -1261,7 +1374,7 @@ impl NFSState {
         SCLogDebug!("parse_udp_ts ({})", input.len());
         if input.len() > 0 {
             match parse_rpc_udp_request(input) {
-                IResult::Done(_, ref rpc_record) => {
+                Ok((_, ref rpc_record)) => {
                     self.is_udp = true;
                     match rpc_record.progver {
                         3 => {
@@ -1273,10 +1386,10 @@ impl NFSState {
                         _ => { status = 1; },
                     }
                 },
-                IResult::Incomplete(_) => {
-                },
-                IResult::Error(_e) => { SCLogDebug!("Parsing failed: {:?}", _e);
+                Err(nom::Err::Incomplete(_)) => {
                 },
+                Err(nom::Err::Error(_e)) |
+                Err(nom::Err::Failure(_e)) => { SCLogDebug!("Parsing failed: {:?}", _e); }
             }
         }
         status
@@ -1288,15 +1401,14 @@ impl NFSState {
         SCLogDebug!("parse_udp_tc ({})", input.len());
         if input.len() > 0 {
             match parse_rpc_udp_reply(input) {
-                IResult::Done(_, ref rpc_record) => {
+                Ok((_, ref rpc_record)) => {
                     self.is_udp = true;
                     status |= self.process_reply_record(rpc_record);
                 },
-                IResult::Incomplete(_) => {
-                },
-                IResult::Error(_e) => {
-                    SCLogDebug!("Parsing failed: {:?}", _e);
+                Err(nom::Err::Incomplete(_)) => {
                 },
+                Err(nom::Err::Error(_e)) |
+                Err(nom::Err::Failure(_e)) => { SCLogDebug!("Parsing failed: {:?}", _e); }
             }
         };
         status
@@ -1321,7 +1433,7 @@ impl NFSState {
 
 /// Returns *mut NFSState
 #[no_mangle]
-pub extern "C" fn rs_nfs_state_new() -> *mut libc::c_void {
+pub extern "C" fn rs_nfs_state_new() -> *mut std::os::raw::c_void {
     let state = NFSState::new();
     let boxed = Box::new(state);
     SCLogDebug!("allocating state");
@@ -1331,7 +1443,7 @@ pub extern "C" fn rs_nfs_state_new() -> *mut libc::c_void {
 /// Params:
 /// - state: *mut NFSState as void pointer
 #[no_mangle]
-pub extern "C" fn rs_nfs_state_free(state: *mut libc::c_void) {
+pub extern "C" fn rs_nfs_state_free(state: *mut std::os::raw::c_void) {
     // Just unbox...
     SCLogDebug!("freeing state");
     let mut nfs_state: Box<NFSState> = unsafe{transmute(state)};
@@ -1340,17 +1452,18 @@ pub extern "C" fn rs_nfs_state_free(state: *mut libc::c_void) {
 
 /// C binding parse a NFS TCP request. Returns 1 on success, -1 on failure.
 #[no_mangle]
-pub extern "C" fn rs_nfs_parse_request(_flow: *mut Flow,
+pub extern "C" fn rs_nfs_parse_request(flow: &mut Flow,
                                        state: &mut NFSState,
-                                       _pstate: *mut libc::c_void,
-                                       input: *mut u8,
+                                       _pstate: *mut std::os::raw::c_void,
+                                       input: *const u8,
                                        input_len: u32,
-                                       _data: *mut libc::c_void)
+                                       _data: *mut std::os::raw::c_void)
                                        -> i8
 {
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
     SCLogDebug!("parsing {} bytes of request data", input_len);
 
+    state.ts = flow.get_last_time().as_secs();
     if state.parse_tcp_data_ts(buf) == 0 {
         1
     } else {
@@ -1371,17 +1484,18 @@ pub extern "C" fn rs_nfs_parse_request_tcp_gap(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_nfs_parse_response(_flow: *mut Flow,
+pub extern "C" fn rs_nfs_parse_response(flow: &mut Flow,
                                         state: &mut NFSState,
-                                        _pstate: *mut libc::c_void,
-                                        input: *mut u8,
+                                        _pstate: *mut std::os::raw::c_void,
+                                        input: *const u8,
                                         input_len: u32,
-                                        _data: *mut libc::c_void)
+                                        _data: *mut std::os::raw::c_void)
                                         -> i8
 {
     SCLogDebug!("parsing {} bytes of response data", input_len);
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
 
+    state.ts = flow.get_last_time().as_secs();
     if state.parse_tcp_data_tc(buf) == 0 {
         1
     } else {
@@ -1405,10 +1519,10 @@ pub extern "C" fn rs_nfs_parse_response_tcp_gap(
 #[no_mangle]
 pub extern "C" fn rs_nfs_parse_request_udp(_flow: *mut Flow,
                                        state: &mut NFSState,
-                                       _pstate: *mut libc::c_void,
-                                       input: *mut u8,
+                                       _pstate: *mut std::os::raw::c_void,
+                                       input: *const u8,
                                        input_len: u32,
-                                       _data: *mut libc::c_void)
+                                       _data: *mut std::os::raw::c_void)
                                        -> i8
 {
     let buf = unsafe{std::slice::from_raw_parts(input, input_len as usize)};
@@ -1424,10 +1538,10 @@ pub extern "C" fn rs_nfs_parse_request_udp(_flow: *mut Flow,
 #[no_mangle]
 pub extern "C" fn rs_nfs_parse_response_udp(_flow: *mut Flow,
                                         state: &mut NFSState,
-                                        _pstate: *mut libc::c_void,
-                                        input: *mut u8,
+                                        _pstate: *mut std::os::raw::c_void,
+                                        input: *const u8,
                                         input_len: u32,
-                                        _data: *mut libc::c_void)
+                                        _data: *mut std::os::raw::c_void)
                                         -> i8
 {
     SCLogDebug!("parsing {} bytes of response data", input_len);
@@ -1493,7 +1607,7 @@ pub extern "C" fn rs_nfs_state_tx_free(state: &mut NFSState,
 #[no_mangle]
 pub extern "C" fn rs_nfs_state_progress_completion_status(
     _direction: u8)
-    -> libc::c_int
+    -> std::os::raw::c_int
 {
     return 1;
 }
@@ -1583,23 +1697,37 @@ pub extern "C" fn rs_nfs_tx_get_detect_flags(
 }
 
 #[no_mangle]
-pub extern "C" fn rs_nfs_state_get_events(state: &mut NFSState,
-                                          tx_id: u64)
+pub extern "C" fn rs_nfs_state_get_events(tx: *mut std::os::raw::c_void)
                                           -> *mut AppLayerDecoderEvents
 {
-    match state.get_tx_by_id(tx_id) {
-        Some(tx) => {
-            return tx.events;
-        }
-        _ => {
-            return std::ptr::null_mut();
-        }
-    }
+    let tx = cast_pointer!(tx, NFSTransaction);
+    return tx.events;
 }
 
 #[no_mangle]
-pub extern "C" fn rs_nfs_state_get_event_info(event_name: *const libc::c_char,
-                                              event_id: *mut libc::c_int,
+pub extern "C" fn rs_nfs_state_get_event_info_by_id(event_id: std::os::raw::c_int,
+                                              event_name: *mut *const std::os::raw::c_char,
+                                              event_type: *mut AppLayerEventType)
+                                              -> i8
+{
+    if let Some(e) = NFSEvent::from_i32(event_id as i32) {
+        let estr = match e {
+            NFSEvent::MalformedData => { "malformed_data\0" },
+            NFSEvent::NonExistingVersion => { "non_existing_version\0" },
+            NFSEvent::UnsupportedVersion => { "unsupported_version\0" },
+        };
+        unsafe{
+            *event_name = estr.as_ptr() as *const std::os::raw::c_char;
+            *event_type = APP_LAYER_EVENT_TYPE_TRANSACTION;
+        };
+        0
+    } else {
+        -1
+    }
+}
+#[no_mangle]
+pub extern "C" fn rs_nfs_state_get_event_info(event_name: *const std::os::raw::c_char,
+                                              event_id: *mut std::os::raw::c_int,
                                               event_type: *mut AppLayerEventType)
                                               -> i8
 {
@@ -1618,7 +1746,7 @@ pub extern "C" fn rs_nfs_state_get_event_info(event_name: *const libc::c_char,
     };
     unsafe{
         *event_type = APP_LAYER_EVENT_TYPE_TRANSACTION;
-        *event_id = event as libc::c_int;
+        *event_id = event as std::os::raw::c_int;
     };
     0
 }
@@ -1675,10 +1803,30 @@ pub extern "C" fn rs_nfs_init(context: &'static mut SuricataFileContext)
     }
 }
 
+fn nfs_probe_dir(i: &[u8], rdir: *mut u8) -> i8 {
+    match parse_rpc_packet_header(i) {
+        Ok((_, ref hdr)) => {
+            let dir = if hdr.msgtype == 0 {
+                STREAM_TOSERVER
+            } else {
+                STREAM_TOCLIENT
+            };
+            unsafe { *rdir = dir };
+            return 1;
+        },
+        Err(nom::Err::Incomplete(_)) => {
+            return 0;
+        },
+        Err(_) => {
+            return -1;
+        },
+    }
+}
+
 pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
     if direction == STREAM_TOCLIENT {
         match parse_rpc_reply(i) {
-            IResult::Done(_, ref rpc) => {
+            Ok((_, ref rpc)) => {
                 if rpc.hdr.frag_len >= 24 && rpc.hdr.frag_len <= 35000 && rpc.hdr.msgtype == 1 && rpc.reply_state == 0 && rpc.accept_state == 0 {
                     SCLogDebug!("TC PROBE LEN {} XID {} TYPE {}", rpc.hdr.frag_len, rpc.hdr.xid, rpc.hdr.msgtype);
                     return 1;
@@ -1686,9 +1834,9 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
                     return -1;
                 }
             },
-            IResult::Incomplete(_) => {
+            Err(nom::Err::Incomplete(_)) => {
                 match parse_rpc_packet_header (i) {
-                    IResult::Done(_, ref rpc_hdr) => {
+                    Ok((_, ref rpc_hdr)) => {
                         if rpc_hdr.frag_len >= 24 && rpc_hdr.frag_len <= 35000 && rpc_hdr.xid != 0 && rpc_hdr.msgtype == 1 {
                             SCLogDebug!("TC PROBE LEN {} XID {} TYPE {}", rpc_hdr.frag_len, rpc_hdr.xid, rpc_hdr.msgtype);
                             return 1;
@@ -1696,8 +1844,8 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
                             return -1;
                         }
                     },
-                    IResult::Incomplete(_) => { },
-                    IResult::Error(_) => {
+                    Err(nom::Err::Incomplete(_)) => { },
+                    Err(_) => {
                         return -1;
                     },
                 }
@@ -1705,13 +1853,13 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
 
                 return 0;
             },
-            IResult::Error(_) => {
+            Err(_) => {
                 return -1;
             },
         }
     } else {
         match parse_rpc(i) {
-            IResult::Done(_, ref rpc) => {
+            Ok((_, ref rpc)) => {
                 if rpc.hdr.frag_len >= 40 && rpc.hdr.msgtype == 0 &&
                    rpc.rpcver == 2 && (rpc.progver == 3 || rpc.progver == 4) &&
                    rpc.program == 100003 &&
@@ -1722,10 +1870,10 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
                     return -1;
                 }
             },
-            IResult::Incomplete(_) => {
+            Err(nom::Err::Incomplete(_)) => {
                 return 0;
             },
-            IResult::Error(_) => {
+            Err(_) => {
                 return -1;
             },
         }
@@ -1735,7 +1883,7 @@ pub fn nfs_probe(i: &[u8], direction: u8) -> i8 {
 pub fn nfs_probe_udp(i: &[u8], direction: u8) -> i8 {
     if direction == STREAM_TOCLIENT {
         match parse_rpc_udp_reply(i) {
-            IResult::Done(_, ref rpc) => {
+            Ok((_, ref rpc)) => {
                 if i.len() >= 32 && rpc.hdr.msgtype == 1 && rpc.reply_state == 0 && rpc.accept_state == 0 {
                     SCLogDebug!("TC PROBE LEN {} XID {} TYPE {}", rpc.hdr.frag_len, rpc.hdr.xid, rpc.hdr.msgtype);
                     return 1;
@@ -1743,16 +1891,13 @@ pub fn nfs_probe_udp(i: &[u8], direction: u8) -> i8 {
                     return -1;
                 }
             },
-            IResult::Incomplete(_) => {
-                return -1;
-            },
-            IResult::Error(_) => {
+            Err(_) => {
                 return -1;
             },
         }
     } else {
         match parse_rpc_udp_request(i) {
-            IResult::Done(_, ref rpc) => {
+            Ok((_, ref rpc)) => {
                 if i.len() >= 48 && rpc.hdr.msgtype == 0 && rpc.progver == 3 && rpc.program == 100003 {
                     return 1;
                 } else if i.len() >= 48 && rpc.hdr.msgtype == 0 && rpc.progver == 2 && rpc.program == 100003 {
@@ -1762,36 +1907,56 @@ pub fn nfs_probe_udp(i: &[u8], direction: u8) -> i8 {
                     return -1;
                 }
             },
-            IResult::Incomplete(_) => {
-                return -1;
-            },
-            IResult::Error(_) => {
+            Err(_) => {
                 return -1;
             },
         }
     }
 }
 
-/// TOSERVER probe function
+/// MIDSTREAM
 #[no_mangle]
-pub extern "C" fn rs_nfs_probe_ts(
-        input: *const u8,
-        len: u32) -> i8
+pub extern "C" fn rs_nfs_probe_ms(
+        direction: u8, input: *const u8,
+        len: u32, rdir: *mut u8) -> i8
 {
-    let slice: &[u8] = unsafe {
-        std::slice::from_raw_parts(input as *mut u8, len as usize)
-    };
-    return nfs_probe(slice, STREAM_TOSERVER);
+    let slice: &[u8] = build_slice!(input, len as usize);
+    SCLogDebug!("rs_nfs_probe_ms: probing direction {:02x}", direction);
+    let mut adirection : u8 = 0;
+    match nfs_probe_dir(slice, &mut adirection) {
+        1 => {
+            if adirection == STREAM_TOSERVER {
+                SCLogDebug!("nfs_probe_dir said STREAM_TOSERVER");
+            } else {
+                SCLogDebug!("nfs_probe_dir said STREAM_TOCLIENT");
+            }
+            let r = nfs_probe(slice, adirection);
+            if r == 1 {
+                SCLogDebug!("nfs_probe success: dir {:02x} adir {:02x}", direction, adirection);
+                if (direction & (STREAM_TOSERVER|STREAM_TOCLIENT)) != adirection {
+                    unsafe { *rdir = adirection; }
+                }
+                return 1;
+            }
+            return r;
+        },
+        0 => {
+            return 0;
+        },
+        _ => {
+            return -1;
+        }
+    }
 }
 
-/// TOCLIENT probe function
 #[no_mangle]
-pub extern "C" fn rs_nfs_probe_tc(input: *const u8, len: u32) -> i8
+pub extern "C" fn rs_nfs_probe(direction: u8,
+        input: *const u8, len: u32)
+    -> i8
 {
-    let slice: &[u8] = unsafe {
-        std::slice::from_raw_parts(input as *mut u8, len as usize)
-    };
-    return nfs_probe(slice, STREAM_TOCLIENT);
+    let slice: &[u8] = build_slice!(input, len as usize);
+    SCLogDebug!("rs_nfs_probe: running probe");
+    return nfs_probe(slice, direction);
 }
 
 /// TOSERVER probe function
@@ -1799,9 +1964,7 @@ pub extern "C" fn rs_nfs_probe_tc(input: *const u8, len: u32) -> i8
 pub extern "C" fn rs_nfs_probe_udp_ts(input: *const u8, len: u32)
                                -> i8
 {
-    let slice: &[u8] = unsafe {
-        std::slice::from_raw_parts(input as *mut u8, len as usize)
-    };
+    let slice: &[u8] = build_slice!(input, len as usize);
     return nfs_probe_udp(slice, STREAM_TOSERVER);
 }
 
@@ -1810,9 +1973,7 @@ pub extern "C" fn rs_nfs_probe_udp_ts(input: *const u8, len: u32)
 pub extern "C" fn rs_nfs_probe_udp_tc(input: *const u8, len: u32)
                                -> i8
 {
-    let slice: &[u8] = unsafe {
-        std::slice::from_raw_parts(input as *mut u8, len as usize)
-    };
+    let slice: &[u8] = build_slice!(input, len as usize);
     return nfs_probe_udp(slice, STREAM_TOCLIENT);
 }
 
