@@ -20,6 +20,7 @@
  *
  * \author Pablo Rincon Crespo <pablo.rincon.crespo@gmail.com>
  * \author Eric Leblond <eric@regit.org>
+ * \author Jeff Lucovsky <jeff@lucovsky.org>
  *
  * App Layer Parser for FTP
  */
@@ -49,6 +50,7 @@
 #include "app-layer-expectation.h"
 
 #include "util-spm.h"
+#include "util-mpm.h"
 #include "util-unittest.h"
 #include "util-debug.h"
 #include "util-memcmp.h"
@@ -57,15 +59,80 @@
 #include "util-mem.h"
 #include "util-misc.h"
 
-#ifdef HAVE_RUST
 #include "rust-ftp-mod-gen.h"
-#endif
 
+#include "output-json.h"
+
+typedef struct FTPThreadCtx_ {
+    MpmThreadCtx *ftp_mpm_thread_ctx;
+    PrefilterRuleStore *pmq;
+} FTPThreadCtx;
+
+#define FTP_MPM mpm_default_matcher
+
+static MpmCtx *ftp_mpm_ctx = NULL;
+
+const FtpCommand FtpCommands[FTP_COMMAND_MAX + 1] = {
+    /* Parsed and handled */
+    { FTP_COMMAND_PORT, "PORT", 4},
+    { FTP_COMMAND_EPRT, "EPRT", 4},
+    { FTP_COMMAND_AUTH_TLS, "AUTH TLS", 8},
+    { FTP_COMMAND_PASV, "PASV", 4},
+    { FTP_COMMAND_RETR, "RETR", 4},
+    { FTP_COMMAND_EPSV, "EPSV", 4},
+    { FTP_COMMAND_STOR, "STOR", 4},
+
+    /* Parsed, but not handled */
+    { FTP_COMMAND_ABOR, "ABOR", 4},
+    { FTP_COMMAND_ACCT, "ACCT", 4},
+    { FTP_COMMAND_ALLO, "ALLO", 4},
+    { FTP_COMMAND_APPE, "APPE", 4},
+    { FTP_COMMAND_CDUP, "CDUP", 4},
+    { FTP_COMMAND_CHMOD, "CHMOD", 5},
+    { FTP_COMMAND_CWD, "CWD", 3},
+    { FTP_COMMAND_DELE, "DELE", 4},
+    { FTP_COMMAND_HELP, "HELP", 4},
+    { FTP_COMMAND_IDLE, "IDLE", 4},
+    { FTP_COMMAND_LIST, "LIST", 4},
+    { FTP_COMMAND_MAIL, "MAIL", 4},
+    { FTP_COMMAND_MDTM, "MDTM", 4},
+    { FTP_COMMAND_MKD, "MKD", 3},
+    { FTP_COMMAND_MLFL, "MLFL", 4},
+    { FTP_COMMAND_MODE, "MODE", 4},
+    { FTP_COMMAND_MRCP, "MRCP", 4},
+    { FTP_COMMAND_MRSQ, "MRSQ", 4},
+    { FTP_COMMAND_MSAM, "MSAM", 4},
+    { FTP_COMMAND_MSND, "MSND", 4},
+    { FTP_COMMAND_MSOM, "MSOM", 4},
+    { FTP_COMMAND_NLST, "NLST", 4},
+    { FTP_COMMAND_NOOP, "NOOP", 4},
+    { FTP_COMMAND_PASS, "PASS", 4},
+    { FTP_COMMAND_PWD, "PWD", 3},
+    { FTP_COMMAND_QUIT, "QUIT", 4},
+    { FTP_COMMAND_REIN, "REIN", 4},
+    { FTP_COMMAND_REST, "REST", 4},
+    { FTP_COMMAND_RMD, "RMD", 3},
+    { FTP_COMMAND_RNFR, "RNFR", 4},
+    { FTP_COMMAND_RNTO, "RNTO", 4},
+    { FTP_COMMAND_SITE, "SITE", 4},
+    { FTP_COMMAND_SIZE, "SIZE", 4},
+    { FTP_COMMAND_SMNT, "SMNT", 4},
+    { FTP_COMMAND_STAT, "STAT", 4},
+    { FTP_COMMAND_STOU, "STOU", 4},
+    { FTP_COMMAND_STRU, "STRU", 4},
+    { FTP_COMMAND_SYST, "SYST", 4},
+    { FTP_COMMAND_TYPE, "TYPE", 4},
+    { FTP_COMMAND_UMASK, "UMASK", 5},
+    { FTP_COMMAND_USER, "USER", 4},
+    { FTP_COMMAND_UNKNOWN, NULL, 0}
+};
 uint64_t ftp_config_memcap = 0;
 
 SC_ATOMIC_DECLARE(uint64_t, ftp_memuse);
 SC_ATOMIC_DECLARE(uint64_t, ftp_memcap);
 
+static FTPTransaction *FTPGetOldestTx(FtpState *);
+
 static void FTPParseMemcap(void)
 {
     const char *conf_val;
@@ -188,6 +255,99 @@ static void FTPFree(void *ptr, size_t size)
     FTPDecrMemuse((uint64_t)size);
 }
 
+static FTPString *FTPStringAlloc(void)
+{
+    return FTPCalloc(1, sizeof(FTPString));
+}
+
+static void FTPStringFree(FTPString *str)
+{
+    if (str->str) {
+        FTPFree(str->str, str->len);
+    }
+
+    FTPFree(str, sizeof(FTPString));
+}
+
+static void *FTPLocalStorageAlloc(void)
+{
+    /* needed by the mpm */
+    FTPThreadCtx *td = SCCalloc(1, sizeof(*td));
+    if (td == NULL) {
+        exit(EXIT_FAILURE);
+    }
+
+    td->pmq = SCCalloc(1, sizeof(*td->pmq));
+    if (td->pmq == NULL) {
+        exit(EXIT_FAILURE);
+    }
+    PmqSetup(td->pmq);
+
+    td->ftp_mpm_thread_ctx = SCCalloc(1, sizeof(MpmThreadCtx));
+    if (unlikely(td->ftp_mpm_thread_ctx == NULL)) {
+        exit(EXIT_FAILURE);
+    }
+    MpmInitThreadCtx(td->ftp_mpm_thread_ctx, FTP_MPM);
+    return td;
+}
+
+static void FTPLocalStorageFree(void *ptr)
+{
+    FTPThreadCtx *td = ptr;
+    if (td != NULL) {
+        if (td->pmq != NULL) {
+            PmqFree(td->pmq);
+            SCFree(td->pmq);
+        }
+
+        if (td->ftp_mpm_thread_ctx != NULL) {
+            mpm_table[FTP_MPM].DestroyThreadCtx(ftp_mpm_ctx, td->ftp_mpm_thread_ctx);
+            SCFree(td->ftp_mpm_thread_ctx);
+        }
+
+        SCFree(td);
+    }
+
+    return;
+}
+static FTPTransaction *FTPTransactionCreate(FtpState *state)
+{
+    SCEnter();
+    FTPTransaction *tx = FTPCalloc(1, sizeof(*tx));
+    if (tx == NULL) {
+        return NULL;
+    }
+
+    TAILQ_INSERT_TAIL(&state->tx_list, tx, next);
+    tx->tx_id = state->tx_cnt++;
+
+    TAILQ_INIT(&tx->response_list);
+
+    SCLogDebug("new transaction %p (state tx cnt %"PRIu64")", tx, state->tx_cnt);
+    return tx;
+}
+
+static void FTPTransactionFree(FTPTransaction *tx)
+{
+    SCEnter();
+
+    if (tx->de_state != NULL) {
+        DetectEngineStateFree(tx->de_state);
+    }
+
+    if (tx->request) {
+        FTPFree(tx->request, tx->request_length);
+    }
+
+    FTPString *str = NULL;
+    while ((str = TAILQ_FIRST(&tx->response_list))) {
+        TAILQ_REMOVE(&tx->response_list, str, next);
+        FTPStringFree(str);
+    }
+
+    SCFree(tx);
+}
+
 static int FTPGetLineForDirection(FtpState *state, FtpLineState *line_state)
 {
     void *ptmp;
@@ -310,45 +470,32 @@ static int FTPGetLine(FtpState *state)
 
 /**
  * \brief This function is called to determine and set which command is being
- * transfered to the ftp server
- * \param ftp_state the ftp state structure for the parser
+ * transferred to the ftp server
+ * \param thread context
  * \param input input line of the command
  * \param len of the command
+ * \param cmd_descriptor when the command has been parsed
  *
  * \retval 1 when the command is parsed, 0 otherwise
  */
-static int FTPParseRequestCommand(void *ftp_state, uint8_t *input,
-                                  uint32_t input_len)
+static int FTPParseRequestCommand(FTPThreadCtx *td,
+                                  const uint8_t *input, uint32_t input_len,
+                                  const FtpCommand **cmd_descriptor)
 {
     SCEnter();
-    FtpState *fstate = (FtpState *)ftp_state;
-    fstate->command = FTP_COMMAND_UNKNOWN;
 
-    if (input_len >= 4 && SCMemcmpLowercase("port", input, 4) == 0) {
-        fstate->command = FTP_COMMAND_PORT;
+    /* I don't like this pmq reset here.  We'll devise a method later, that
+     * should make the use of the mpm very efficient */
+    PmqReset(td->pmq);
+    int mpm_cnt = mpm_table[FTP_MPM].Search(ftp_mpm_ctx, td->ftp_mpm_thread_ctx,
+                                            td->pmq, input, input_len);
+    if (mpm_cnt) {
+        *cmd_descriptor = &FtpCommands[td->pmq->rule_id_array[0]];
+        SCReturnInt(1);
     }
 
-    if (input_len >= 8 && SCMemcmpLowercase("auth tls", input, 8) == 0) {
-        fstate->command = FTP_COMMAND_AUTH_TLS;
-    }
-
-    if (input_len >= 4 && SCMemcmpLowercase("pasv", input, 4) == 0) {
-        fstate->command = FTP_COMMAND_PASV;
-    }
-
-    if (input_len > 5 && SCMemcmpLowercase("retr", input, 4) == 0) {
-        fstate->command = FTP_COMMAND_RETR;
-    }
-
-    if (input_len >= 4 && SCMemcmpLowercase("epsv", input, 4) == 0) {
-        fstate->command = FTP_COMMAND_EPSV;
-    }
-
-    if (input_len > 5 && SCMemcmpLowercase("stor", input, 4) == 0) {
-        fstate->command = FTP_COMMAND_STOR;
-    }
-
-    return 1;
+    *cmd_descriptor = NULL;
+    SCReturnInt(0);
 }
 
 struct FtpTransferCmd {
@@ -367,11 +514,33 @@ static void FtpTransferCmdFree(void *data)
     if (cmd == NULL)
         return;
     if (cmd->file_name) {
-        SCFree(cmd->file_name);
+        FTPFree(cmd->file_name, cmd->file_len);
     }
     FTPFree(cmd, sizeof(struct FtpTransferCmd));
 }
 
+static uint32_t CopyCommandLine(uint8_t **dest, const uint8_t *src, uint32_t length)
+{
+    if (likely(length)) {
+        uint8_t *where = FTPCalloc(length + 1, sizeof(char));
+        if (unlikely(where == NULL)) {
+            return 0;
+        }
+        memcpy(where, src, length);
+
+        /* Remove trailing newlines/carriage returns */
+        while (length && isspace((unsigned char) where[length - 1])) {
+            length--;
+        }
+
+        where[length] = '\0';
+        *dest = where;
+    }
+    /* either 0 or actual */
+    return length;
+}
+
+
 /**
  * \brief This function is called to retrieve a ftp request
  * \param ftp_state the ftp state structure for the parser
@@ -383,9 +552,11 @@ static void FtpTransferCmdFree(void *data)
  */
 static int FTPParseRequest(Flow *f, void *ftp_state,
                            AppLayerParserState *pstate,
-                           uint8_t *input, uint32_t input_len,
+                           const uint8_t *input, uint32_t input_len,
                            void *local_data, const uint8_t flags)
 {
+    FTPThreadCtx *thread_data = local_data;
+
     SCEnter();
     /* PrintRawDataFp(stdout, input,input_len); */
 
@@ -405,21 +576,40 @@ static int FTPParseRequest(Flow *f, void *ftp_state,
 
     int direction = STREAM_TOSERVER;
     while (FTPGetLine(state) >= 0) {
-        FTPParseRequestCommand(state,
-                               state->current_line, state->current_line_len);
+        const FtpCommand *cmd_descriptor;
+
+        if (!FTPParseRequestCommand(thread_data, state->current_line, state->current_line_len, &cmd_descriptor)) {
+            state->command = FTP_COMMAND_UNKNOWN;
+            continue;
+        }
+
+        state->command = cmd_descriptor->command;
+
+        FTPTransaction *tx = FTPTransactionCreate(state);
+        if (unlikely(tx == NULL))
+            return -1;
+        state->curr_tx = tx;
+
+        tx->command_descriptor = cmd_descriptor;
+        tx->request_length = CopyCommandLine(&tx->request, state->current_line, state->current_line_len);
+
         switch (state->command) {
+            case FTP_COMMAND_EPRT:
+                // fallthrough
             case FTP_COMMAND_PORT:
-                if (state->current_line_len > state->port_line_size) {
+                if (state->current_line_len + 1 > state->port_line_size) {
+                    /* Allocate an extra byte for a NULL terminator */
                     ptmp = FTPRealloc(state->port_line, state->port_line_size,
                                       state->current_line_len);
                     if (ptmp == NULL) {
-                        FTPFree(state->port_line, state->port_line_size);
-                        state->port_line = NULL;
-                        state->port_line_size = 0;
+                        if (state->port_line) {
+                            FTPFree(state->port_line, state->port_line_size);
+                            state->port_line = NULL;
+                            state->port_line_size = 0;
+                        }
                         return 0;
                     }
                     state->port_line = ptmp;
-
                     state->port_line_size = state->current_line_len;
                 }
                 memcpy(state->port_line, state->current_line,
@@ -434,8 +624,10 @@ static int FTPParseRequest(Flow *f, void *ftp_state,
                 // fallthrough
             case FTP_COMMAND_STOR:
                 {
-                    /* No dyn port negotiated so get out */
-                    if (state->dyn_port == 0) {
+                    /* Ensure that there is a negotiated dyn port and a file
+                     * name -- need more than 5 chars: cmd [4], space, <filename>
+                     */
+                    if (state->dyn_port == 0 || state->current_line_len < 6) {
                         SCReturnInt(-1);
                     }
                     struct FtpTransferCmd *data = FTPCalloc(1, sizeof(struct FtpTransferCmd));
@@ -445,7 +637,7 @@ static int FTPParseRequest(Flow *f, void *ftp_state,
                     /* Min size has been checked in FTPParseRequestCommand */
                     data->file_name = FTPCalloc(state->current_line_len - 4, sizeof(char));
                     if (data->file_name == NULL) {
-                        FTPFree(data, sizeof(struct FtpTransferCmd));
+                        FtpTransferCmdFree(data);
                         SCReturnInt(-1);
                     }
                     data->file_name[state->current_line_len - 5] = 0;
@@ -453,18 +645,23 @@ static int FTPParseRequest(Flow *f, void *ftp_state,
                     memcpy(data->file_name, state->current_line + 5, state->current_line_len - 5);
                     data->cmd = state->command;
                     data->flow_id = FlowGetId(f);
-                    int ret = AppLayerExpectationCreate(f, direction, 0,
-                                                         state->dyn_port,
-                                                         ALPROTO_FTPDATA, data);
+                    int ret = AppLayerExpectationCreate(f,
+                                            state->active ? STREAM_TOSERVER : direction,
+                                            0, state->dyn_port, ALPROTO_FTPDATA, data);
                     if (ret == -1) {
-                        FTPFree(data, sizeof(struct FtpTransferCmd));
+                        FtpTransferCmdFree(data);
                         SCLogDebug("No expectation created.");
                         SCReturnInt(-1);
                     } else {
-                        SCLogDebug("Expectation created.");
+                        SCLogDebug("Expectation created [direction: %s, dynamic port %"PRIu16"].",
+                            state->active ? "to server" : "to client",
+                            state->dyn_port);
                     }
+
                     /* reset the dyn port to avoid duplicate */
                     state->dyn_port = 0;
+                    /* reset active/passive indicator */
+                    state->active = false;
                 }
                 break;
             default:
@@ -475,60 +672,50 @@ static int FTPParseRequest(Flow *f, void *ftp_state,
     return 1;
 }
 
-static int FTPParsePassiveResponse(Flow *f, FtpState *state, uint8_t *input, uint32_t input_len)
+static int FTPParsePassiveResponse(Flow *f, FtpState *state, const uint8_t *input, uint32_t input_len)
 {
-    uint16_t dyn_port;
-
-#ifdef HAVE_RUST
-    dyn_port = rs_ftp_pasv_response(input, input_len);
+    uint16_t dyn_port = rs_ftp_pasv_response(input, input_len);
     if (dyn_port == 0) {
         return -1;
     }
-#else
-    uint16_t part1, part2;
-    uint8_t *ptr = memrchr(input, ',', input_len);
-    if (ptr == NULL)
-        return -1;
-
-    part2 = atoi((char *)ptr + 1);
-    ptr = memrchr(input, ',', (ptr - input) - 1);
-    if (ptr == NULL)
-        return -1;
-    part1 = atoi((char *)ptr + 1);
-
-    dyn_port = 256 * part1 + part2;
-#endif
+    SCLogDebug("FTP passive mode (v4): dynamic port %"PRIu16"", dyn_port);
+    state->active = false;
     state->dyn_port = dyn_port;
+    state->curr_tx->dyn_port = dyn_port;
+    state->curr_tx->active = false;
 
     return 0;
 }
 
-static int FTPParsePassiveResponseV6(Flow *f, FtpState *state, uint8_t *input, uint32_t input_len)
+static int FTPParsePassiveResponseV6(Flow *f, FtpState *state, const uint8_t *input, uint32_t input_len)
 {
-#ifdef HAVE_RUST
     uint16_t dyn_port = rs_ftp_epsv_response(input, input_len);
     if (dyn_port == 0) {
         return -1;
     }
-
+    SCLogDebug("FTP passive mode (v6): dynamic port %"PRIu16"", dyn_port);
+    state->active = false;
     state->dyn_port = dyn_port;
-#else
-    uint8_t *ptr = memrchr(input, '|', input_len);
-    if (ptr == NULL) {
-        return -1;
-    } else {
-        int n_length =  ptr - input - 1;
-        if (n_length < 4)
-            return -1;
-        ptr = memrchr(input, '|', n_length);
-        if (ptr == NULL)
-            return -1;
-    }
-    state->dyn_port = atoi((char *)ptr + 1);
-#endif
+    state->curr_tx->dyn_port = dyn_port;
+    state->curr_tx->active = false;
     return 0;
 }
 
+/**
+ * \brief  Handle preliminary replies -- keep tx open
+ * \retval: True for a positive preliminary reply; false otherwise
+ *
+ * 1yz   Positive Preliminary reply
+ *
+ *                The requested action is being initiated; expect another
+ *                               reply before proceeding with a new command
+ */
+static inline bool FTPIsPPR(const uint8_t *input, uint32_t input_len)
+{
+    return input_len >= 4 && isdigit(input[0]) && input[0] == '1' &&
+           isdigit(input[1]) && isdigit(input[2]) && isspace(input[3]);
+}
+
 /**
  * \brief This function is called to retrieve a ftp response
  * \param ftp_state the ftp state structure for the parser
@@ -539,17 +726,63 @@ static int FTPParsePassiveResponseV6(Flow *f, FtpState *state, uint8_t *input, u
  * \retval 1 when the command is parsed, 0 otherwise
  */
 static int FTPParseResponse(Flow *f, void *ftp_state, AppLayerParserState *pstate,
-                            uint8_t *input, uint32_t input_len,
+                            const uint8_t *input, uint32_t input_len,
                             void *local_data, const uint8_t flags)
 {
     FtpState *state = (FtpState *)ftp_state;
+    int retcode = 1;
 
+    if (unlikely(input_len == 0)) {
+        return 1;
+    }
+
+    FTPTransaction *tx = FTPGetOldestTx(state);
+    if (tx == NULL) {
+        tx = FTPTransactionCreate(state);
+    }
+    if (unlikely(tx == NULL)) {
+        return -1;
+    }
+    if (state->command == FTP_COMMAND_UNKNOWN || tx->command_descriptor == NULL) {
+        /* unknown */
+        tx->command_descriptor = &FtpCommands[FTP_COMMAND_MAX -1];
+    }
+
+    state->curr_tx = tx;
     if (state->command == FTP_COMMAND_AUTH_TLS) {
         if (input_len >= 4 && SCMemcmp("234 ", input, 4) == 0) {
             AppLayerRequestProtocolTLSUpgrade(f);
         }
     }
 
+    if (state->command == FTP_COMMAND_EPRT) {
+        uint16_t dyn_port = rs_ftp_active_eprt(state->port_line, state->port_line_len);
+        if (dyn_port == 0) {
+            retcode = 0;
+            goto tx_complete;
+        }
+        state->dyn_port = dyn_port;
+        state->active = true;
+        tx->dyn_port = dyn_port;
+        tx->active = true;
+        SCLogDebug("FTP active mode (v6): dynamic port %"PRIu16"", dyn_port);
+    }
+
+    if (state->command == FTP_COMMAND_PORT) {
+        if ((flags & STREAM_TOCLIENT)) {
+            uint16_t dyn_port = rs_ftp_active_port(state->port_line, state->port_line_len);
+            if (dyn_port == 0) {
+                retcode = 0;
+                goto tx_complete;
+            }
+            state->dyn_port = dyn_port;
+            state->active = true;
+            tx->dyn_port = state->dyn_port;
+            tx->active = true;
+            SCLogDebug("FTP active mode (v4): dynamic port %"PRIu16"", dyn_port);
+        }
+    }
+
     if (state->command == FTP_COMMAND_PASV) {
         if (input_len >= 4 && SCMemcmp("227 ", input, 4) == 0) {
             FTPParsePassiveResponse(f, ftp_state, input, input_len);
@@ -562,9 +795,25 @@ static int FTPParseResponse(Flow *f, void *ftp_state, AppLayerParserState *pstat
         }
     }
 
-    return 1;
+    if (likely(input_len)) {
+        FTPString *response = FTPStringAlloc();
+        if (likely(response)) {
+            response->len = CopyCommandLine(&response->str, input, input_len);
+            TAILQ_INSERT_TAIL(&tx->response_list, response, next);
+        }
+    }
+
+    /* Handle preliminary replies -- keep tx open */
+    if (FTPIsPPR(input, input_len)) {
+        return retcode;
+    }
+
+tx_complete:
+    tx->done = true;
+    return retcode;
 }
 
+
 #ifdef DEBUG
 static SCMutex ftp_state_mem_lock = SCMUTEX_INITIALIZER;
 static uint64_t ftp_state_memuse = 0;
@@ -573,11 +822,12 @@ static uint64_t ftp_state_memcnt = 0;
 
 static void *FTPStateAlloc(void)
 {
-    void *s = FTPMalloc(sizeof(FtpState));
+    void *s = FTPCalloc(1, sizeof(FtpState));
     if (unlikely(s == NULL))
         return NULL;
 
-    memset(s, 0, sizeof(FtpState));
+    FtpState *ftp_state = (FtpState *) s;
+    TAILQ_INIT(&ftp_state->tx_list);
 
 #ifdef DEBUG
     SCMutexLock(&ftp_state_mem_lock);
@@ -600,8 +850,14 @@ static void FTPStateFree(void *s)
 
     //AppLayerDecoderEventsFreeEvents(&s->decoder_events);
 
-    if (fstate->de_state != NULL) {
-        DetectEngineStateFree(fstate->de_state);
+    FTPTransaction *tx = NULL;
+    while ((tx = TAILQ_FIRST(&fstate->tx_list))) {
+        TAILQ_REMOVE(&fstate->tx_list, tx, next);
+        SCLogDebug("[%s] state %p id %"PRIu64", Freeing %d bytes at %p",
+            tx->command_descriptor->command_name,
+            s, tx->tx_id,
+            tx->request_length, tx->request);
+        FTPTransactionFree(tx);
     }
 
     FTPFree(s, sizeof(FtpState));
@@ -615,32 +871,114 @@ static void FTPStateFree(void *s)
 
 static int FTPSetTxDetectState(void *vtx, DetectEngineState *de_state)
 {
-    FtpState *ftp_state = (FtpState *)vtx;
-    ftp_state->de_state = de_state;
+    FTPTransaction *tx = (FTPTransaction *)vtx;
+    tx->de_state = de_state;
     return 0;
 }
 
-static DetectEngineState *FTPGetTxDetectState(void *vtx)
+/**
+ * \brief This function returns the oldest open transaction; if none
+ * are open, then the oldest transaction is returned
+ * \param ftp_state the ftp state structure for the parser
+ *
+ * \retval transaction pointer when a transaction was found; NULL otherwise.
+ */
+static FTPTransaction *FTPGetOldestTx(FtpState *ftp_state)
 {
-    FtpState *ftp_state = (FtpState *)vtx;
-    return ftp_state->de_state;
-}
-
-static void FTPStateTransactionFree(void *state, uint64_t tx_id)
-{
-    /* do nothing */
+    if (unlikely(!ftp_state)) {
+        SCLogDebug("NULL state object; no transactions available");
+        return NULL;
+    }
+    FTPTransaction *tx = NULL;
+    FTPTransaction *lasttx = NULL;
+    TAILQ_FOREACH(tx, &ftp_state->tx_list, next) {
+        /* Return oldest open tx */
+        if (!tx->done) {
+            SCLogDebug("Returning tx %p id %"PRIu64, tx, tx->tx_id);
+            return tx;
+        }
+        /* save for the end */
+        lasttx = tx;
+    }
+    /* All tx are closed; return last element */
+    if (lasttx)
+        SCLogDebug("Returning OLDEST tx %p id %"PRIu64, lasttx, lasttx->tx_id);
+    return lasttx;
 }
 
 static void *FTPGetTx(void *state, uint64_t tx_id)
 {
     FtpState *ftp_state = (FtpState *)state;
-    return ftp_state;
+    if (ftp_state) {
+        FTPTransaction *tx = NULL;
+
+        if (ftp_state->curr_tx == NULL)
+            return NULL;
+        if (ftp_state->curr_tx->tx_id == tx_id)
+            return ftp_state->curr_tx;
+
+        TAILQ_FOREACH(tx, &ftp_state->tx_list, next) {
+            if (tx->tx_id == tx_id)
+                return tx;
+        }
+    }
+    return NULL;
+}
+
+static DetectEngineState *FTPGetTxDetectState(void *vtx)
+{
+    FTPTransaction *tx = (FTPTransaction *)vtx;
+    return tx->de_state;
+}
+
+
+static uint64_t FTPGetTxDetectFlags(void *vtx, uint8_t dir)
+{
+    FTPTransaction *tx = (FTPTransaction *)vtx;
+    if (dir & STREAM_TOSERVER) {
+        return tx->detect_flags_ts;
+    } else {
+        return tx->detect_flags_tc;
+    }
+}
+
+static void FTPSetTxDetectFlags(void *vtx, uint8_t dir, uint64_t flags)
+{
+    FTPTransaction *tx = (FTPTransaction *)vtx;
+    if (dir & STREAM_TOSERVER) {
+        tx->detect_flags_ts = flags;
+    } else {
+        tx->detect_flags_tc = flags;
+    }
+}
+
+static void FTPStateTransactionFree(void *state, uint64_t tx_id)
+{
+    FtpState *ftp_state = state;
+    FTPTransaction *tx = NULL;
+    TAILQ_FOREACH(tx, &ftp_state->tx_list, next) {
+        if (tx_id < tx->tx_id)
+            break;
+        else if (tx_id > tx->tx_id)
+            continue;
+
+        if (tx == ftp_state->curr_tx)
+            ftp_state->curr_tx = NULL;
+        TAILQ_REMOVE(&ftp_state->tx_list, tx, next);
+        FTPTransactionFree(tx);
+        break;
+    }
 }
 
 static uint64_t FTPGetTxCnt(void *state)
 {
-    /* single tx */
-    return 1;
+    uint64_t cnt = 0;
+    FtpState *ftp_state = state;
+    if (ftp_state) {
+        cnt = ftp_state->tx_cnt;
+    }
+    SCLogDebug("returning state %p %"PRIu64, state, cnt);
+    return cnt;
 }
 
 static int FTPGetAlstateProgressCompletionStatus(uint8_t direction)
@@ -648,18 +986,20 @@ static int FTPGetAlstateProgressCompletionStatus(uint8_t direction)
     return FTP_STATE_FINISHED;
 }
 
-static int FTPGetAlstateProgress(void *tx, uint8_t direction)
+static int FTPGetAlstateProgress(void *vtx, uint8_t direction)
 {
-    FtpState *ftp_state = (FtpState *)tx;
+    SCLogDebug("tx %p", vtx);
+    FTPTransaction *tx = vtx;
 
     if (direction == STREAM_TOSERVER &&
-        ftp_state->command == FTP_COMMAND_PORT) {
+        tx->command_descriptor->command == FTP_COMMAND_PORT) {
         return FTP_STATE_PORT_DONE;
     }
 
-    /* TODO: figure out further progress handling */
+    if (!tx->done)
+        return FTP_STATE_IN_PROGRESS;
 
-    return FTP_STATE_IN_PROGRESS;
+    return FTP_STATE_FINISHED;
 }
 
 
@@ -708,7 +1048,7 @@ static StreamingBufferConfig sbcfg = STREAMING_BUFFER_CONFIG_INITIALIZER;
  */
 static int FTPDataParse(Flow *f, FtpDataState *ftpdata_state,
         AppLayerParserState *pstate,
-        uint8_t *input, uint32_t input_len,
+        const uint8_t *input, uint32_t input_len,
         void *local_data, int direction)
 {
     uint16_t flags = FileFlowToFlags(f, direction);
@@ -780,15 +1120,23 @@ static int FTPDataParse(Flow *f, FtpDataState *ftpdata_state,
     }
 
 out:
-    if (ftpdata_state->files) {
-        FilePrune(ftpdata_state->files);
-    }
     return ret;
 }
 
+static void FTPStateSetTxLogged(void *state, void *vtx, LoggerId logged)
+{
+    FTPTransaction *tx = vtx;
+    tx->logged = logged;
+}
+
+static LoggerId FTPStateGetTxLogged(void *state, void *vtx)
+{
+    FTPTransaction *tx = vtx;
+    return tx->logged;
+}
 static int FTPDataParseRequest(Flow *f, void *ftp_state,
         AppLayerParserState *pstate,
-        uint8_t *input, uint32_t input_len,
+        const uint8_t *input, uint32_t input_len,
         void *local_data, const uint8_t flags)
 {
     return FTPDataParse(f, ftp_state, pstate, input, input_len,
@@ -797,7 +1145,7 @@ static int FTPDataParseRequest(Flow *f, void *ftp_state,
 
 static int FTPDataParseResponse(Flow *f, void *ftp_state,
         AppLayerParserState *pstate,
-        uint8_t *input, uint32_t input_len,
+        const uint8_t *input, uint32_t input_len,
         void *local_data, const uint8_t flags)
 {
     return FTPDataParse(f, ftp_state, pstate, input, input_len,
@@ -812,12 +1160,12 @@ static uint64_t ftpdata_state_memcnt = 0;
 
 static void *FTPDataStateAlloc(void)
 {
-    void *s = FTPMalloc(sizeof(FtpDataState));
+    void *s = FTPCalloc(1, sizeof(FtpDataState));
     if (unlikely(s == NULL))
         return NULL;
 
-    memset(s, 0, sizeof(FtpDataState));
-    ((FtpDataState *)s)->state = FTPDATA_STATE_IN_PROGRESS;
+    FtpDataState *state = (FtpDataState *) s;
+    state->state = FTPDATA_STATE_IN_PROGRESS;
 
 #ifdef DEBUG
     SCMutexLock(&ftpdata_state_mem_lock);
@@ -863,6 +1211,26 @@ static DetectEngineState *FTPDataGetTxDetectState(void *vtx)
     return ftp_state->de_state;
 }
 
+static void FTPDataSetTxDetectFlags(void *vtx, uint8_t dir, uint64_t flags)
+{
+    FtpDataState *ftp_state = (FtpDataState *)vtx;
+    if (dir & STREAM_TOSERVER) {
+        ftp_state->detect_flags_ts = flags;
+    } else {
+        ftp_state->detect_flags_tc = flags;
+    }
+}
+
+static uint64_t FTPDataGetTxDetectFlags(void *vtx, uint8_t dir)
+{
+    FtpDataState *ftp_state = (FtpDataState *)vtx;
+    if (dir & STREAM_TOSERVER) {
+        return ftp_state->detect_flags_ts;
+    } else {
+        return ftp_state->detect_flags_tc;
+    }
+}
+
 static void FTPDataStateTransactionFree(void *state, uint64_t tx_id)
 {
     /* do nothing */
@@ -901,6 +1269,41 @@ static FileContainer *FTPDataStateGetFiles(void *state, uint8_t direction)
     SCReturnPtr(ftpdata_state->files, "FileContainer");
 }
 
+static void FTPSetMpmState(void)
+{
+    ftp_mpm_ctx = SCMalloc(sizeof(MpmCtx));
+    if (unlikely(ftp_mpm_ctx == NULL)) {
+        exit(EXIT_FAILURE);
+    }
+    memset(ftp_mpm_ctx, 0, sizeof(MpmCtx));
+    MpmInitCtx(ftp_mpm_ctx, FTP_MPM);
+
+    uint32_t i = 0;
+    for (i = 0; i < sizeof(FtpCommands)/sizeof(FtpCommand) - 1; i++) {
+        const FtpCommand *cmd = &FtpCommands[i];
+        if (cmd->command_length == 0)
+            continue;
+
+        MpmAddPatternCI(ftp_mpm_ctx,
+                       (uint8_t *)cmd->command_name,
+                       cmd->command_length,
+                       0 /* defunct */, 0 /* defunct */,
+                       i /*  id */, i /* rule id */ , 0 /* no flags */);
+    }
+
+    mpm_table[FTP_MPM].Prepare(ftp_mpm_ctx);
+
+}
+
+static void FTPFreeMpmState(void)
+{
+    if (ftp_mpm_ctx != NULL) {
+        mpm_table[FTP_MPM].DestroyCtx(ftp_mpm_ctx);
+        SCFree(ftp_mpm_ctx);
+        ftp_mpm_ctx = NULL;
+    }
+}
+
 void RegisterFTPParsers(void)
 {
     const char *proto_name = "ftp";
@@ -927,8 +1330,15 @@ void RegisterFTPParsers(void)
         AppLayerParserRegisterDetectStateFuncs(IPPROTO_TCP, ALPROTO_FTP,
                 FTPGetTxDetectState, FTPSetTxDetectState);
 
-        AppLayerParserRegisterGetTx(IPPROTO_TCP, ALPROTO_FTP, FTPGetTx);
+        AppLayerParserRegisterDetectFlagsFuncs(IPPROTO_TCP, ALPROTO_FTP,
+                                               FTPGetTxDetectFlags, FTPSetTxDetectFlags);
 
+        AppLayerParserRegisterGetTx(IPPROTO_TCP, ALPROTO_FTP, FTPGetTx);
+        AppLayerParserRegisterLoggerFuncs(IPPROTO_TCP, ALPROTO_FTP, FTPStateGetTxLogged,
+                                          FTPStateSetTxLogged);
+
+        AppLayerParserRegisterLocalStorageFunc(IPPROTO_TCP, ALPROTO_FTP, FTPLocalStorageAlloc,
+                                               FTPLocalStorageFree);
         AppLayerParserRegisterGetTxCnt(IPPROTO_TCP, ALPROTO_FTP, FTPGetTxCnt);
 
         AppLayerParserRegisterGetStateProgressFunc(IPPROTO_TCP, ALPROTO_FTP, FTPGetAlstateProgress);
@@ -947,6 +1357,8 @@ void RegisterFTPParsers(void)
         AppLayerParserRegisterTxFreeFunc(IPPROTO_TCP, ALPROTO_FTPDATA, FTPDataStateTransactionFree);
         AppLayerParserRegisterDetectStateFuncs(IPPROTO_TCP, ALPROTO_FTPDATA,
                 FTPDataGetTxDetectState, FTPDataSetTxDetectState);
+        AppLayerParserRegisterDetectFlagsFuncs(IPPROTO_TCP, ALPROTO_FTPDATA,
+                FTPDataGetTxDetectFlags, FTPDataSetTxDetectFlags);
 
         AppLayerParserRegisterGetFilesFunc(IPPROTO_TCP, ALPROTO_FTPDATA, FTPDataStateGetFiles);
 
@@ -970,6 +1382,9 @@ void RegisterFTPParsers(void)
         SCLogInfo("Parsed disabled for %s protocol. Protocol detection"
                   "still on.", proto_name);
     }
+
+    FTPSetMpmState();
+
 #ifdef UNITTESTS
     AppLayerParserRegisterProtocolUnittests(IPPROTO_TCP, ALPROTO_FTP, FTPParserRegisterTests);
 #endif
@@ -986,7 +1401,33 @@ void FTPAtExitPrintStats(void)
 }
 
 
-#ifdef HAVE_LIBJANSSON
+/*
+ * \brief Returns the ending offset of the next line from a multi-line buffer.
+ *
+ * "Buffer" refers to a FTP response in a single buffer containing multiple lines.
+ * Here, "next line" is defined as terminating on
+ * - Newline character
+ * - Null character
+ *
+ * \param buffer Contains zero or more characters.
+ * \param len Size, in bytes, of buffer.
+ *
+ * \retval Offset from the start of buffer indicating the where the
+ * next "line ends". The characters between the input buffer and this
+ * value comprise the line.
+ *
+ * NULL is found first or a newline isn't found, then UINT16_MAX is returned.
+ */
+uint16_t JsonGetNextLineFromBuffer(const char *buffer, const uint16_t len)
+{
+    if (!buffer || *buffer == '\0') {
+        return UINT16_MAX;
+    }
+
+    char *c = strchr(buffer, '\n');
+    return c == NULL ? len : c - buffer + 1;
+}
+
 json_t *JsonFTPDataAddMetadata(const Flow *f)
 {
     const FtpDataState *ftp_state = NULL;
@@ -997,10 +1438,10 @@ json_t *JsonFTPDataAddMetadata(const Flow *f)
     if (ftpd == NULL)
         return NULL;
     if (ftp_state->file_name) {
-        char *s = BytesToString(ftp_state->file_name, ftp_state->file_len);
-        json_object_set_new(ftpd, "filename", json_string(s));
-        if (s != NULL)
-            SCFree(s);
+        size_t size = ftp_state->file_len * 2 + 1;
+        char string[size];
+        BytesToStringBuffer(ftp_state->file_name, ftp_state->file_len, string, size);
+        json_object_set_new(ftpd, "filename", SCJsonString(string));
     }
     switch (ftp_state->command) {
         case FTP_COMMAND_STOR:
@@ -1014,7 +1455,14 @@ json_t *JsonFTPDataAddMetadata(const Flow *f)
     }
     return ftpd;
 }
-#endif /* HAVE_LIBJANSSON */
+
+/**
+ * \brief Free memory allocated for global SMTP parser state.
+ */
+void FTPParserCleanup(void)
+{
+    FTPFreeMpmState();
+}
 
 /* UNITTESTS */
 #ifdef UNITTESTS
@@ -1071,7 +1519,7 @@ end:
     return result;
 }
 
-/** \test Send a splitted get request. */
+/** \test Send a split get request. */
 static int FTPParserTest03(void)
 {
     int result = 1;
@@ -1324,6 +1772,166 @@ static int FTPParserTest10(void)
         goto end;
     }
 
+end:
+    if (alp_tctx != NULL)
+        AppLayerParserThreadCtxFree(alp_tctx);
+    StreamTcpFreeConfig(TRUE);
+    FLOW_DESTROY(&f);
+    return result;
+}
+
+/** \test Supply RETR without a filename */
+static int FTPParserTest11(void)
+{
+    int result = 1;
+    Flow f;
+    uint8_t ftpbuf1[] = "PORT 192,168,1,1,0,80\r\n";
+    uint8_t ftpbuf2[] = "RETR\r\n";
+    uint8_t ftpbuf3[] = "227 OK\r\n";
+    TcpSession ssn;
+
+    AppLayerParserThreadCtx *alp_tctx = AppLayerParserThreadCtxAlloc();
+
+    memset(&f, 0, sizeof(f));
+    memset(&ssn, 0, sizeof(ssn));
+
+    FLOW_INITIALIZE(&f);
+    f.protoctx = (void *)&ssn;
+    f.proto = IPPROTO_TCP;
+    f.alproto = ALPROTO_FTP;
+
+    StreamTcpInitConfig(TRUE);
+
+    FLOWLOCK_WRLOCK(&f);
+    int r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOSERVER | STREAM_START, ftpbuf1,
+                                sizeof(ftpbuf1) - 1);
+    if (r != 0) {
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    /* Response */
+    FLOWLOCK_WRLOCK(&f);
+    r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOCLIENT,
+                                ftpbuf3,
+                                sizeof(ftpbuf3) - 1);
+    if (r != 0) {
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    FLOWLOCK_WRLOCK(&f);
+    r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOSERVER, ftpbuf2,
+                                sizeof(ftpbuf2) - 1);
+    if (r == 0) {
+        SCLogDebug("parse should've failed");
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    FtpState *ftp_state = f.alstate;
+    if (ftp_state == NULL) {
+        SCLogDebug("no ftp state: ");
+        result = 0;
+        goto end;
+    }
+
+    if (ftp_state->command != FTP_COMMAND_RETR) {
+        SCLogDebug("expected command %" PRIu32 ", got %" PRIu32 ": ",
+                   FTP_COMMAND_RETR, ftp_state->command);
+        result = 0;
+        goto end;
+    }
+
+end:
+    if (alp_tctx != NULL)
+        AppLayerParserThreadCtxFree(alp_tctx);
+    StreamTcpFreeConfig(TRUE);
+    FLOW_DESTROY(&f);
+    return result;
+}
+
+/** \test Supply STOR without a filename */
+static int FTPParserTest12(void)
+{
+    int result = 1;
+    Flow f;
+    uint8_t ftpbuf1[] = "PORT 192,168,1,1,0,80\r\n";
+    uint8_t ftpbuf2[] = "STOR\r\n";
+    uint8_t ftpbuf3[] = "227 OK\r\n";
+    TcpSession ssn;
+
+    AppLayerParserThreadCtx *alp_tctx = AppLayerParserThreadCtxAlloc();
+
+    memset(&f, 0, sizeof(f));
+    memset(&ssn, 0, sizeof(ssn));
+
+    FLOW_INITIALIZE(&f);
+    f.protoctx = (void *)&ssn;
+    f.proto = IPPROTO_TCP;
+    f.alproto = ALPROTO_FTP;
+
+    StreamTcpInitConfig(TRUE);
+
+    FLOWLOCK_WRLOCK(&f);
+    int r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOSERVER | STREAM_START, ftpbuf1,
+                                sizeof(ftpbuf1) - 1);
+    if (r != 0) {
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    /* Response */
+    FLOWLOCK_WRLOCK(&f);
+    r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOCLIENT,
+                                ftpbuf3,
+                                sizeof(ftpbuf3) - 1);
+    if (r != 0) {
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    FLOWLOCK_WRLOCK(&f);
+    r = AppLayerParserParse(NULL, alp_tctx, &f, ALPROTO_FTP,
+                                STREAM_TOSERVER, ftpbuf2,
+                                sizeof(ftpbuf2) - 1);
+    if (r == 0) {
+        SCLogDebug("parse should've failed");
+        result = 0;
+        FLOWLOCK_UNLOCK(&f);
+        goto end;
+    }
+    FLOWLOCK_UNLOCK(&f);
+
+    FtpState *ftp_state = f.alstate;
+    if (ftp_state == NULL) {
+        SCLogDebug("no ftp state: ");
+        result = 0;
+        goto end;
+    }
+
+    if (ftp_state->command != FTP_COMMAND_STOR) {
+        SCLogDebug("expected command %" PRIu32 ", got %" PRIu32 ": ",
+                   FTP_COMMAND_STOR, ftp_state->command);
+        result = 0;
+        goto end;
+    }
+
 end:
     if (alp_tctx != NULL)
         AppLayerParserThreadCtxFree(alp_tctx);
@@ -1341,6 +1949,8 @@ void FTPParserRegisterTests(void)
     UtRegisterTest("FTPParserTest06", FTPParserTest06);
     UtRegisterTest("FTPParserTest07", FTPParserTest07);
     UtRegisterTest("FTPParserTest10", FTPParserTest10);
+    UtRegisterTest("FTPParserTest11", FTPParserTest11);
+    UtRegisterTest("FTPParserTest12", FTPParserTest12);
 #endif /* UNITTESTS */
 }
 
