@@ -100,7 +100,7 @@ static char stats_enabled = TRUE;
 
 /**< add decoder events as stats? enabled by default */
 bool stats_decoder_events = true;
-const char *stats_decoder_events_prefix = "decoder";
+const char *stats_decoder_events_prefix = "decoder.event";
 /**< add stream events as stats? disabled by default */
 bool stats_stream_events = false;
 
@@ -116,6 +116,11 @@ static int stats_loggers_active = 1;
 
 static uint16_t counters_global_id = 0;
 
+bool StatsEnabled(void)
+{
+    return (stats_enabled == TRUE);
+}
+
 static void StatsPublicThreadContextInit(StatsPublicThreadContext *t)
 {
     SCMutexInit(&t->m, NULL);
@@ -240,6 +245,14 @@ static void StatsInitCtxPreOutput(void)
             SCLogDebug("Stats module has been disabled");
             SCReturn;
         }
+        /* warn if we are using legacy config to enable stats */
+        ConfNode *gstats = ConfGetNode("stats");
+        if (gstats == NULL) {
+            SCLogWarning(SC_ERR_STATS_LOG_GENERIC, "global stats config is missing. "
+                    "Stats enabled through legacy stats.log. "
+                    "See %s%s/configuration/suricata-yaml.html#stats", DOC_URL, DOC_VERSION);
+        }
+
         const char *interval = ConfNodeLookupChildValue(stats, "interval");
         if (interval != NULL)
             stats_tts = (uint32_t) atoi(interval);
@@ -256,12 +269,7 @@ static void StatsInitCtxPreOutput(void)
 
         const char *prefix = NULL;
         if (ConfGet("stats.decoder-events-prefix", &prefix) != 1) {
-            prefix = "decoder";
-            SCLogWarning(SC_WARN_DEFAULT_WILL_CHANGE, "in 5.0 the default "
-                    "for decoder event stats will go from "
-                    "'decoder.<proto>.<event>' to 'decoder.event.<proto>.<event>'. "
-                    "See ticket #2225. To suppress this message, "
-                    "set stats.decoder-events-prefix in the yaml.");
+            prefix = "decoder.event";
         }
         stats_decoder_events_prefix = prefix;
     }
@@ -280,7 +288,7 @@ static void StatsInitCtxPostOutput(void)
         exit(EXIT_FAILURE);
     }
 
-    if (!OutputStatsLoggersRegistered()) {
+    if (stats_enabled && !OutputStatsLoggersRegistered()) {
         stats_loggers_active = 0;
 
         /* if the unix command socket is enabled we do the background
@@ -356,7 +364,6 @@ static void StatsReleaseCtx(void)
 static void *StatsMgmtThread(void *arg)
 {
     ThreadVars *tv_local = (ThreadVars *)arg;
-    uint8_t run = 1;
 
     /* Set the thread name */
     if (SCSetThreadName(tv_local->name) < 0) {
@@ -368,7 +375,6 @@ static void *StatsMgmtThread(void *arg)
 
     /* Set the threads capability */
     tv_local->cap_flags = 0;
-
     SCDropCaps(tv_local);
 
     if (stats_ctx == NULL) {
@@ -390,7 +396,7 @@ static void *StatsMgmtThread(void *arg)
     SCLogDebug("stats_thread_data %p", &stats_thread_data);
 
     TmThreadsSetFlag(tv_local, THV_INIT_DONE);
-    while (run) {
+    while (1) {
         if (TmThreadsCheckFlag(tv_local, THV_PAUSE)) {
             TmThreadsSetFlag(tv_local, THV_PAUSED);
             TmThreadTestThreadUnPaused(tv_local);
@@ -399,7 +405,6 @@ static void *StatsMgmtThread(void *arg)
 
         struct timeval cur_timev;
         gettimeofday(&cur_timev, NULL);
-
         struct timespec cond_time = FROM_TIMEVAL(cur_timev);
         cond_time.tv_sec += (stats_tts);
 
@@ -414,7 +419,7 @@ static void *StatsMgmtThread(void *arg)
         SCMutexUnlock(&stats_table_mutex);
 
         if (TmThreadsCheckFlag(tv_local, THV_KILL)) {
-            run = 0;
+            break;
         }
     }
 
@@ -442,10 +447,6 @@ static void *StatsMgmtThread(void *arg)
 static void *StatsWakeupThread(void *arg)
 {
     ThreadVars *tv_local = (ThreadVars *)arg;
-    uint8_t run = 1;
-    ThreadVars *tv = NULL;
-    PacketQueue *q = NULL;
-    struct timespec cond_time;
 
     /* Set the thread name */
     if (SCSetThreadName(tv_local->name) < 0) {
@@ -457,7 +458,6 @@ static void *StatsWakeupThread(void *arg)
 
     /* Set the threads capability */
     tv_local->cap_flags = 0;
-
     SCDropCaps(tv_local);
 
     if (stats_ctx == NULL) {
@@ -468,15 +468,17 @@ static void *StatsWakeupThread(void *arg)
     }
 
     TmThreadsSetFlag(tv_local, THV_INIT_DONE);
-    while (run) {
+    while (1) {
         if (TmThreadsCheckFlag(tv_local, THV_PAUSE)) {
             TmThreadsSetFlag(tv_local, THV_PAUSED);
             TmThreadTestThreadUnPaused(tv_local);
             TmThreadsUnsetFlag(tv_local, THV_PAUSED);
         }
 
-        cond_time.tv_sec = time(NULL) + STATS_WUT_TTS;
-        cond_time.tv_nsec = 0;
+        struct timeval cur_timev;
+        gettimeofday(&cur_timev, NULL);
+        struct timespec cond_time = FROM_TIMEVAL(cur_timev);
+        cond_time.tv_sec += STATS_WUT_TTS;
 
         /* wait for the set time, or until we are woken up by
          * the shutdown procedure */
@@ -484,7 +486,8 @@ static void *StatsWakeupThread(void *arg)
         SCCtrlCondTimedwait(tv_local->ctrl_cond, tv_local->ctrl_mutex, &cond_time);
         SCCtrlMutexUnlock(tv_local->ctrl_mutex);
 
-        tv = tv_root[TVT_PPT];
+        SCMutexLock(&tv_root_lock);
+        ThreadVars *tv = tv_root[TVT_PPT];
         while (tv != NULL) {
             if (tv->perf_public_ctx.head == NULL) {
                 tv = tv->next;
@@ -496,7 +499,7 @@ static void *StatsWakeupThread(void *arg)
             tv->perf_public_ctx.perf_flag = 1;
 
             if (tv->inq != NULL) {
-                q = &trans_q[tv->inq->id];
+                PacketQueue *q = &trans_q[tv->inq->id];
                 SCCondSignal(&q->cond_q);
             }
 
@@ -517,15 +520,15 @@ static void *StatsWakeupThread(void *arg)
 
             tv = tv->next;
         }
+        SCMutexUnlock(&tv_root_lock);
 
         if (TmThreadsCheckFlag(tv_local, THV_KILL)) {
-            run = 0;
+            break;
         }
     }
 
     TmThreadsSetFlag(tv_local, THV_RUNNING_DONE);
     TmThreadWaitForFlag(tv_local, THV_DEINIT);
-
     TmThreadsSetFlag(tv_local, THV_CLOSED);
     return NULL;
 }
@@ -686,7 +689,7 @@ static int StatsOutput(ThreadVars *tv)
 
         SCLogDebug("Thread %d %s ctx %p", thread, sts->name, sts->ctx);
 
-        /* temporay table for quickly storing the counters for this
+        /* temporary table for quickly storing the counters for this
          * thread store, so that we can post process them outside
          * of the thread store lock */
         struct CountersMergeTable thread_table[max_id];
@@ -718,8 +721,7 @@ static int StatsOutput(ThreadVars *tv)
         SCMutexUnlock(&sts->ctx->m);
 
         /* update merge table */
-        uint16_t c;
-        for (c = 0; c < max_id; c++) {
+        for (uint16_t c = 0; c < max_id; c++) {
             struct CountersMergeTable *e = &thread_table[c];
             /* thread only sets type if it has a counter
              * of this type. */
@@ -744,7 +746,7 @@ static int StatsOutput(ThreadVars *tv)
         }
 
         /* update per thread stats table */
-        for (c = 0; c < max_id; c++) {
+        for (uint16_t c = 0; c < max_id; c++) {
             struct CountersMergeTable *e = &thread_table[c];
             /* thread only sets type if it has a counter
              * of this type. */
@@ -776,8 +778,7 @@ static int StatsOutput(ThreadVars *tv)
     }
 
     /* transfer 'merge table' to final stats table */
-    uint16_t x;
-    for (x = 0; x < max_id; x++) {
+    for (uint16_t x = 0; x < max_id; x++) {
         /* xfer previous value to pvalue and reset value */
         table[x].pvalue = table[x].value;
         table[x].value = 0;
@@ -816,15 +817,19 @@ TmEcode StatsOutputCounterSocket(json_t *cmd,
     json_t *message = NULL;
     TmEcode r = TM_ECODE_OK;
 
-    SCMutexLock(&stats_table_mutex);
-    if (stats_table.start_time == 0) {
+    if (!stats_enabled) {
         r = TM_ECODE_FAILED;
-        message = json_string("stats not yet synchronized");
+        message = json_string("stats are disabled in the config");
     } else {
-        message = StatsToJSON(&stats_table, JSON_STATS_TOTALS|JSON_STATS_THREADS);
+        SCMutexLock(&stats_table_mutex);
+        if (stats_table.start_time == 0) {
+            r = TM_ECODE_FAILED;
+            message = json_string("stats not yet synchronized");
+        } else {
+            message = StatsToJSON(&stats_table, JSON_STATS_TOTALS|JSON_STATS_THREADS);
+        }
+        SCMutexUnlock(&stats_table_mutex);
     }
-    SCMutexUnlock(&stats_table_mutex);
-
     json_object_set_new(answer, "message", message);
     return r;
 }
@@ -836,8 +841,7 @@ static void StatsLogSummary(void)
     SCMutexLock(&stats_table_mutex);
     if (stats_table.start_time != 0) {
         const StatsTable *st = &stats_table;
-        uint32_t u = 0;
-        for (u = 0; u < st->nstats; u++) {
+        for (uint32_t u = 0; u < st->nstats; u++) {
             const char *name = st->stats[u].name;
             if (name == NULL || strcmp(name, "detect.alert") != 0)
                 continue;
@@ -939,10 +943,9 @@ void StatsSpawnThreads(void)
 uint16_t StatsRegisterCounter(const char *name, struct ThreadVars_ *tv)
 {
     uint16_t id = StatsRegisterQualifiedCounter(name,
-                                                 (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
-                                                 &tv->perf_public_ctx,
-                                                 STATS_TYPE_NORMAL, NULL);
-
+            (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
+            &tv->perf_public_ctx,
+            STATS_TYPE_NORMAL, NULL);
     return id;
 }
 
@@ -960,10 +963,9 @@ uint16_t StatsRegisterCounter(const char *name, struct ThreadVars_ *tv)
 uint16_t StatsRegisterAvgCounter(const char *name, struct ThreadVars_ *tv)
 {
     uint16_t id = StatsRegisterQualifiedCounter(name,
-                                                 (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
-                                                 &tv->perf_public_ctx,
-                                                 STATS_TYPE_AVERAGE, NULL);
-
+            (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
+            &tv->perf_public_ctx,
+            STATS_TYPE_AVERAGE, NULL);
     return id;
 }
 
@@ -981,10 +983,9 @@ uint16_t StatsRegisterAvgCounter(const char *name, struct ThreadVars_ *tv)
 uint16_t StatsRegisterMaxCounter(const char *name, struct ThreadVars_ *tv)
 {
     uint16_t id = StatsRegisterQualifiedCounter(name,
-                                                 (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
-                                                 &tv->perf_public_ctx,
-                                                 STATS_TYPE_MAXIMUM, NULL);
-
+            (tv->thread_group_name != NULL) ? tv->thread_group_name : tv->printable_name,
+            &tv->perf_public_ctx,
+            STATS_TYPE_MAXIMUM, NULL);
     return id;
 }
 
@@ -1006,9 +1007,9 @@ uint16_t StatsRegisterGlobalCounter(const char *name, uint64_t (*Func)(void))
     BUG_ON(stats_ctx == NULL);
 #endif
     uint16_t id = StatsRegisterQualifiedCounter(name, NULL,
-                                                 &(stats_ctx->global_counter_ctx),
-                                                 STATS_TYPE_FUNC,
-                                                 Func);
+            &(stats_ctx->global_counter_ctx),
+            STATS_TYPE_FUNC,
+            Func);
     return id;
 }
 
@@ -1021,15 +1022,12 @@ static uint32_t CountersIdHashFunc(HashTable *ht, void *data, uint16_t datalen)
 {
     CountersIdType *t = (CountersIdType *)data;
     uint32_t hash = 0;
-    int i = 0;
-
     int len = strlen(t->string);
 
-    for (i = 0; i < len; i++)
+    for (int i = 0; i < len; i++)
         hash += tolower((unsigned char)t->string[i]);
 
     hash = hash % ht->array_size;
-
     return hash;
 }
 
@@ -1079,8 +1077,6 @@ static int StatsThreadRegister(const char *thread_name, StatsPublicThreadContext
         return 0;
     }
 
-    StatsThreadStore *temp = NULL;
-
     if (thread_name == NULL || pctx == NULL) {
         SCLogDebug("supplied argument(s) to StatsThreadRegister NULL");
         return 0;
@@ -1109,6 +1105,7 @@ static int StatsThreadRegister(const char *thread_name, StatsPublicThreadContext
     }
 
 
+    StatsThreadStore *temp = NULL;
     if ( (temp = SCMalloc(sizeof(StatsThreadStore))) == NULL) {
         SCMutexUnlock(&stats_ctx->sts_lock);
         return 0;
@@ -1208,34 +1205,30 @@ int StatsSetupPrivate(ThreadVars *tv)
 }
 
 /**
- * \brief Syncs the counter array with the global counter variables
+ * \brief the private stats store with the public stats store
  *
  * \param pca      Pointer to the StatsPrivateThreadContext
  * \param pctx     Pointer the the tv's StatsPublicThreadContext
  *
- * \retval  0 on success
+ * \retval  1 on success
  * \retval -1 on error
  */
 int StatsUpdateCounterArray(StatsPrivateThreadContext *pca, StatsPublicThreadContext *pctx)
 {
-    StatsLocalCounter *pcae = NULL;
-    uint32_t i = 0;
 
     if (pca == NULL || pctx == NULL) {
         SCLogDebug("pca or pctx is NULL inside StatsUpdateCounterArray");
         return -1;
     }
 
-    pcae = pca->head;
-
     SCMutexLock(&pctx->m);
-    for (i = 1; i <= pca->size; i++) {
+    StatsLocalCounter *pcae = pca->head;
+    for (uint32_t i = 1; i <= pca->size; i++) {
         StatsCopyCounterValue(&pcae[i]);
     }
     SCMutexUnlock(&pctx->m);
 
     pctx->perf_flag = 0;
-
     return 1;
 }
 
@@ -1264,8 +1257,6 @@ void StatsReleaseResources()
 {
     StatsLogSummary();
     StatsReleaseCtx();
-
-    return;
 }
 
 /**
@@ -1283,8 +1274,6 @@ void StatsReleaseCounters(StatsCounter *head)
         head = head->next;
         StatsReleaseCounter(pc);
     }
-
-    return;
 }
 
 /** \internal
@@ -1303,8 +1292,6 @@ static void StatsReleasePrivateThreadContext(StatsPrivateThreadContext *pca)
         }
         pca->initialized = 0;
     }
-
-    return;
 }
 
 void StatsThreadCleanup(ThreadVars *tv)
