@@ -186,7 +186,7 @@ static int IPV4OptValidateTimestamp(Packet *p, const IPV4Opt *o)
 static int IPV4OptValidateCIPSO(Packet *p, const IPV4Opt *o)
 {
 //    uint32_t doi;
-    uint8_t *tag;
+    const uint8_t *tag;
     uint16_t len;
 
     /* Check length */
@@ -299,7 +299,7 @@ typedef struct IPV4Options_ {
 /**
  * Decode/Validate IPv4 Options.
  */
-static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options *opts)
+static int DecodeIPV4Options(Packet *p, const uint8_t *pkt, uint16_t len, IPV4Options *opts)
 {
     uint16_t plen = len;
 
@@ -374,100 +374,82 @@ static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options
                     if (opts->o_ts.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateTimestamp(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateTimestamp(p, &opt) == 0) {
+                        opts->o_ts = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_TS;
                     }
-                    opts->o_ts = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_TS;
                     break;
                 case IPV4_OPT_RR:
                     if (opts->o_rr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_rr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_RR;
                     }
-                    opts->o_rr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_RR;
                     break;
                 case IPV4_OPT_QS:
                     if (opts->o_qs.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_qs = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_QS;
                     }
-                    opts->o_qs = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_QS;
                     break;
                 case IPV4_OPT_SEC:
                     if (opts->o_sec.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_sec = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SEC;
                     }
-                    opts->o_sec = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SEC;
                     break;
                 case IPV4_OPT_LSRR:
                     if (opts->o_lsrr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_lsrr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_LSRR;
                     }
-                    opts->o_lsrr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_LSRR;
                     break;
                 case IPV4_OPT_CIPSO:
                     if (opts->o_cipso.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateCIPSO(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateCIPSO(p, &opt) == 0) {
+                        opts->o_cipso = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_CIPSO;
                     }
-                    opts->o_cipso = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_CIPSO;
                     break;
                 case IPV4_OPT_SID:
                     if (opts->o_sid.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_sid = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SID;
                     }
-                    opts->o_sid = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SID;
                     break;
                 case IPV4_OPT_SSRR:
                     if (opts->o_ssrr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_ssrr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SSRR;
                     }
-                    opts->o_ssrr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SSRR;
                     break;
                 case IPV4_OPT_RTRALT:
                     if (opts->o_rtralt.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_rtralt = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_RTRALT;
                     }
-                    opts->o_rtralt = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_RTRALT;
                     break;
                 default:
                     SCLogDebug("IPV4OPT <unknown> (%" PRIu8 ") len %" PRIu8,
@@ -485,7 +467,7 @@ static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options
     return 0;
 }
 
-static int DecodeIPV4Packet(Packet *p, uint8_t *pkt, uint16_t len)
+static int DecodeIPV4Packet(Packet *p, const uint8_t *pkt, uint16_t len)
 {
     if (unlikely(len < IPV4_HEADER_LEN)) {
         ENGINE_SET_INVALID_EVENT(p, IPV4_PKT_TOO_SMALL);
@@ -524,13 +506,16 @@ static int DecodeIPV4Packet(Packet *p, uint8_t *pkt, uint16_t len)
     if (ip_opt_len > 0) {
         IPV4Options opts;
         memset(&opts, 0x00, sizeof(opts));
-        DecodeIPV4Options(p, pkt + IPV4_HEADER_LEN, ip_opt_len, &opts);
+        if (DecodeIPV4Options(p, pkt + IPV4_HEADER_LEN, ip_opt_len, &opts) < 0) {
+            return -1;
+        }
     }
 
     return 0;
 }
 
-int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint16_t len, PacketQueue *pq)
+int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
+        const uint8_t *pkt, uint16_t len, PacketQueue *pq)
 {
     StatsIncr(tv, dtv->counter_ipv4);
 
@@ -632,8 +617,8 @@ static int DecodeIPV4OptionsNONETest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
 
     SCFree(p);
     PASS;
@@ -649,8 +634,8 @@ static int DecodeIPV4OptionsEOLTest01(void)
     FAIL_IF(unlikely(p == NULL));
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF (rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     SCFree(p);
     PASS;
 }
@@ -665,8 +650,8 @@ static int DecodeIPV4OptionsNOPTest01(void)
     FAIL_IF(unlikely(p == NULL));
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF (rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     SCFree(p);
     PASS;
 }
@@ -686,8 +671,8 @@ static int DecodeIPV4OptionsRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_rr.type != IPV4_OPT_RR);
     SCFree(p);
     PASS;
@@ -708,8 +693,8 @@ static int DecodeIPV4OptionsRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    FAIL_IF(DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts) != -1);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -730,8 +715,8 @@ static int DecodeIPV4OptionsRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -752,8 +737,8 @@ static int DecodeIPV4OptionsRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -770,8 +755,8 @@ static int DecodeIPV4OptionsQSTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_qs.type != IPV4_OPT_QS);
     SCFree(p);
     PASS;
@@ -788,8 +773,8 @@ static int DecodeIPV4OptionsQSTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_qs.type != 0);
     SCFree(p);
     PASS;
@@ -810,8 +795,8 @@ static int DecodeIPV4OptionsTSTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_ts.type != IPV4_OPT_TS);
     SCFree(p);
     PASS;
@@ -832,8 +817,8 @@ static int DecodeIPV4OptionsTSTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -854,8 +839,8 @@ static int DecodeIPV4OptionsTSTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -876,8 +861,8 @@ static int DecodeIPV4OptionsTSTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -895,8 +880,8 @@ static int DecodeIPV4OptionsSECTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_sec.type != IPV4_OPT_SEC);
     SCFree(p);
     PASS;
@@ -914,8 +899,8 @@ static int DecodeIPV4OptionsSECTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_sec.type != 0);
     SCFree(p);
     PASS;
@@ -936,8 +921,8 @@ static int DecodeIPV4OptionsLSRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_lsrr.type != IPV4_OPT_LSRR);
     SCFree(p);
     PASS;
@@ -958,8 +943,8 @@ static int DecodeIPV4OptionsLSRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -980,8 +965,8 @@ static int DecodeIPV4OptionsLSRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -1002,8 +987,8 @@ static int DecodeIPV4OptionsLSRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -1022,8 +1007,8 @@ static int DecodeIPV4OptionsCIPSOTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_cipso.type != IPV4_OPT_CIPSO);
     SCFree(p);
     PASS;
@@ -1040,8 +1025,8 @@ static int DecodeIPV4OptionsSIDTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_sid.type != IPV4_OPT_SID);
     SCFree(p);
     PASS;
@@ -1058,8 +1043,8 @@ static int DecodeIPV4OptionsSIDTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_sid.type != 0);
     SCFree(p);
     PASS;
@@ -1080,8 +1065,8 @@ static int DecodeIPV4OptionsSSRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_ssrr.type != IPV4_OPT_SSRR);
     SCFree(p);
     PASS;
@@ -1102,8 +1087,8 @@ static int DecodeIPV4OptionsSSRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1124,8 +1109,8 @@ static int DecodeIPV4OptionsSSRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1146,8 +1131,8 @@ static int DecodeIPV4OptionsSSRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1164,8 +1149,8 @@ static int DecodeIPV4OptionsRTRALTTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_rtralt.type != IPV4_OPT_RTRALT);
     SCFree(p);
     PASS;
@@ -1182,8 +1167,8 @@ static int DecodeIPV4OptionsRTRALTTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rtralt.type != 0);
     SCFree(p);
     PASS;
