@@ -54,6 +54,7 @@
 #include "util-unittest-helper.h"
 #include "util-print.h"
 #include "util-profiling.h"
+#include "util-validate.h"
 
 #ifdef OS_WIN32
 #include <winsock.h>
@@ -300,7 +301,7 @@ static int IPOnlyCIDRItemSetup(IPOnlyCIDRItem *gh, char *s)
     /* parse the address */
     if (IPOnlyCIDRItemParseSingle(gh, s) == -1) {
         SCLogError(SC_ERR_ADDRESS_ENGINE_GENERIC,
-                   "DetectAddressParse error \"%s\"", s);
+                   "address parsing error \"%s\"", s);
         goto error;
     }
 
@@ -458,8 +459,8 @@ static void IPOnlyCIDRListPrint(IPOnlyCIDRItem *tmphead)
 
     while (tmphead != NULL) {
         i++;
-        SCLogDebug("Item %"PRIu32" has netmask %"PRIu16" negated:"
-                   " %s; IP: %s; signum: %"PRIu16, i, tmphead->netmask,
+        SCLogDebug("Item %"PRIu32" has netmask %"PRIu8" negated:"
+                   " %s; IP: %s; signum: %"PRIu32, i, tmphead->netmask,
                    (tmphead->negated) ? "yes":"no",
                    inet_ntoa(*(struct in_addr*)&tmphead->ip[0]),
                    tmphead->signum);
@@ -767,7 +768,7 @@ static int IPOnlyCIDRListParse(const DetectEngineCtx *de_ctx,
 
     *gh = IPOnlyCIDRListParse2(de_ctx, str, 0);
     if (*gh == NULL) {
-        SCLogDebug("DetectAddressParse2 returned null");
+        SCLogDebug("IPOnlyCIDRListParse2 returned null");
         goto error;
     }
 
@@ -949,9 +950,9 @@ int IPOnlyMatchCompatSMs(ThreadVars *tv,
     SigMatchData *smd = s->sm_arrays[DETECT_SM_LIST_MATCH];
     if (smd) {
         while (1) {
-            BUG_ON(!(sigmatch_table[smd->type].flags & SIGMATCH_IPONLY_COMPAT));
+            DEBUG_VALIDATE_BUG_ON(!(sigmatch_table[smd->type].flags & SIGMATCH_IPONLY_COMPAT));
             KEYWORD_PROFILING_START;
-            if (sigmatch_table[smd->type].Match(tv, det_ctx, p, s, smd->ctx) > 0) {
+            if (sigmatch_table[smd->type].Match(det_ctx, p, s, smd->ctx) > 0) {
                 KEYWORD_PROFILING_END(det_ctx, smd->type, 1);
                 if (smd->is_last)
                     break;
@@ -983,6 +984,8 @@ void IPOnlyMatchPacket(ThreadVars *tv,
     SigNumArray *dst = NULL;
     void *user_data_src = NULL, *user_data_dst = NULL;
 
+    SCEnter();
+
     if (p->src.family == AF_INET) {
         (void)SCRadixFindKeyIPV4BestMatch((uint8_t *)&GET_IPV4_SRC_ADDR_U32(p),
                                               io_ctx->tree_ipv4src, &user_data_src);
@@ -1003,7 +1006,7 @@ void IPOnlyMatchPacket(ThreadVars *tv,
     dst = user_data_dst;
 
     if (src == NULL || dst == NULL)
-        return;
+        SCReturn;
 
     uint32_t u;
     for (u = 0; u < src->size; u++) {
@@ -1081,7 +1084,7 @@ void IPOnlyMatchPacket(ThreadVars *tv,
                         if (smd != NULL) {
                             while (1) {
                                 KEYWORD_PROFILING_START;
-                                (void)sigmatch_table[smd->type].Match(tv, det_ctx, p, s, smd->ctx);
+                                (void)sigmatch_table[smd->type].Match(det_ctx, p, s, smd->ctx);
                                 KEYWORD_PROFILING_END(det_ctx, smd->type, 1);
                                 if (smd->is_last)
                                     break;
@@ -1102,6 +1105,7 @@ void IPOnlyMatchPacket(ThreadVars *tv,
             }
         }
     }
+    SCReturn;
 }
 
 /**
