@@ -49,7 +49,7 @@
 static pcre *parse_regex;
 static pcre_extra *parse_regex_study;
 
-int DetectFlowbitMatch (ThreadVars *, DetectEngineThreadCtx *, Packet *,
+int DetectFlowbitMatch (DetectEngineThreadCtx *, Packet *,
         const Signature *, const SigMatchCtx *);
 static int DetectFlowbitSetup (DetectEngineCtx *, Signature *, const char *);
 void DetectFlowbitFree (void *);
@@ -123,7 +123,7 @@ static int DetectFlowbitMatchIsnotset (Packet *p, const DetectFlowbitsData *fd)
  *        -1: error
  */
 
-int DetectFlowbitMatch (ThreadVars *t, DetectEngineThreadCtx *det_ctx, Packet *p,
+int DetectFlowbitMatch (DetectEngineThreadCtx *det_ctx, Packet *p,
         const Signature *s, const SigMatchCtx *ctx)
 {
     const DetectFlowbitsData *fd = (const DetectFlowbitsData *)ctx;
@@ -329,11 +329,9 @@ struct FBAnalyze {
     uint32_t toggle_sids_size;
 };
 #ifdef PROFILING
-#ifdef HAVE_LIBJANSSON
 static void DetectFlowbitsAnalyzeDump(const DetectEngineCtx *de_ctx,
         struct FBAnalyze *array, uint32_t elements);
 #endif
-#endif
 
 void DetectFlowbitsAnalyze(DetectEngineCtx *de_ctx)
 {
@@ -520,10 +518,8 @@ void DetectFlowbitsAnalyze(DetectEngineCtx *de_ctx)
         SCFree(varname);
     }
 #ifdef PROFILING
-#ifdef HAVE_LIBJANSSON
     DetectFlowbitsAnalyzeDump(de_ctx, array, array_size);
 #endif
-#endif
 
 end:
     for (uint32_t i = 0; i < array_size; i++) {
@@ -536,7 +532,6 @@ end:
 }
 
 #ifdef PROFILING
-#ifdef HAVE_LIBJANSSON
 #include "output-json.h"
 #include "util-buffer.h"
 SCMutex g_flowbits_dump_write_m = SCMUTEX_INITIALIZER;
@@ -664,7 +659,6 @@ static void DetectFlowbitsAnalyzeDump(const DetectEngineCtx *de_ctx,
     json_object_clear(js);
     json_decref(js);
 }
-#endif /* HAVE_LIBJANSSON */
 #endif /* PROFILING */
 
 #ifdef UNITTESTS
