@@ -55,19 +55,18 @@ static int DetectGeoipSetupNoSupport (DetectEngineCtx *a, Signature *b, const ch
 void DetectGeoipRegister(void)
 {
     sigmatch_table[DETECT_GEOIP].name = "geoip";
+    sigmatch_table[DETECT_GEOIP].desc = "match on the source, destination or source and destination IP addresses of network traffic, and to see to which country it belongs";
+    sigmatch_table[DETECT_GEOIP].url = DOC_URL DOC_VERSION "/rules/header-keywords.html#geoip";
     sigmatch_table[DETECT_GEOIP].Setup = DetectGeoipSetupNoSupport;
     sigmatch_table[DETECT_GEOIP].Free = NULL;
     sigmatch_table[DETECT_GEOIP].RegisterTests = NULL;
 }
 
 #else /* HAVE_GEOIP */
-#ifdef HAVE_LIBGEOIP
-#include <GeoIP.h>
-#else
-#include <maxminddb.h>
-#endif
 
-static int DetectGeoipMatch(ThreadVars *, DetectEngineThreadCtx *, Packet *,
+#include <maxminddb.h>
+
+static int DetectGeoipMatch(DetectEngineThreadCtx *, Packet *,
                             const Signature *, const SigMatchCtx *);
 static int DetectGeoipSetup(DetectEngineCtx *, Signature *, const char *);
 static void DetectGeoipRegisterTests(void);
@@ -80,6 +79,8 @@ static void DetectGeoipDataFree(void *);
 void DetectGeoipRegister(void)
 {
     sigmatch_table[DETECT_GEOIP].name = "geoip";
+    sigmatch_table[DETECT_GEOIP].url = DOC_URL DOC_VERSION "/rules/header-keywords.html#geoip";
+    sigmatch_table[DETECT_GEOIP].desc = "keyword to match on country of src and or dst IP";
     sigmatch_table[DETECT_GEOIP].Match = DetectGeoipMatch;
     sigmatch_table[DETECT_GEOIP].Setup = DetectGeoipSetup;
     sigmatch_table[DETECT_GEOIP].Free = DetectGeoipDataFree;
@@ -94,19 +95,13 @@ void DetectGeoipRegister(void)
  */
 static bool InitGeolocationEngine(DetectGeoipData *geoipdata)
 {
-#ifdef HAVE_LIBGEOIP
-    geoipdata->geoengine = GeoIP_new(GEOIP_MEMORY_CACHE);
-    if (geoipdata->geoengine == NULL)
-        return false;
-    return true;
-#else
     const char *filename = NULL;
 
     /* Get location and name of GeoIP2 database from YAML conf */
     (void)ConfGet("geoip-database", &filename);
 
     if (filename == NULL) {
-        SCLogWarning(SC_ERR_INVALID_ARGUMENT, "Unable to locate a GeoIP2 "
+        SCLogWarning(SC_ERR_INVALID_ARGUMENT, "Unable to locate a GeoIP2"
                      "database filename in YAML conf.  GeoIP rule matching "
                      "is disabled.");
         geoipdata->mmdb_status = MMDB_FILE_OPEN_ERROR;
@@ -126,7 +121,6 @@ static bool InitGeolocationEngine(DetectGeoipData *geoipdata)
                  MMDB_strerror(status));
     geoipdata->mmdb_status = status;
     return false;
-#endif /* HAVE_LIBGEOIP */
 }
 
 /**
@@ -140,10 +134,6 @@ static bool InitGeolocationEngine(DetectGeoipData *geoipdata)
  */
 static const char *GeolocateIPv4(const DetectGeoipData *geoipdata, uint32_t ip)
 {
-#ifdef HAVE_LIBGEOIP
-    if (geoipdata->geoengine != NULL)
-        return GeoIP_country_code_by_ipnum(geoipdata->geoengine,  SCNtohl(ip));
-#else
     int mmdb_error;
     struct sockaddr_in sa;
     sa.sin_family = AF_INET;
@@ -179,7 +169,6 @@ static const char *GeolocateIPv4(const DetectGeoipData *geoipdata, uint32_t ip)
         }
     }
 
-#endif /* HAVE_LIBGEOIP */
     /* The country code for the IP was not found */
     return NULL;
 }
@@ -222,9 +211,7 @@ static int CheckGeoMatchIPv4(const DetectGeoipData *geoipdata, uint32_t ip)
     {
         for (i = 0; i < geoipdata->nlocations; i++) {
             if (strcmp(country, (char *)geoipdata->location[i])==0) {
-#ifndef HAVE_LIBGEOIP
                 SCFree((void *)country);
-#endif
                 return 1;
             }
         }
@@ -232,20 +219,14 @@ static int CheckGeoMatchIPv4(const DetectGeoipData *geoipdata, uint32_t ip)
         /* Check if NEGATED match-on condition */
         for (i = 0; i < geoipdata->nlocations; i++) {
             if (strcmp(country, (char *)geoipdata->location[i])==0) {
-#ifndef HAVE_LIBGEOIP
                 SCFree((void *)country);
-#endif
                 return 0; /* if one matches, rule does NOT match (negated) */
             }
         }
-#ifndef HAVE_LIBGEOIP
         SCFree((void *)country);
-#endif
         return 1; /* returns 1 if no location matches (negated) */
     }
-#ifndef HAVE_LIBGEOIP
     SCFree((void *)country);
-#endif
     return 0;
 }
 
@@ -261,7 +242,7 @@ static int CheckGeoMatchIPv4(const DetectGeoipData *geoipdata, uint32_t ip)
  * \retval 0 no match
  * \retval 1 match
  */
-static int DetectGeoipMatch(ThreadVars *t, DetectEngineThreadCtx *det_ctx,
+static int DetectGeoipMatch(DetectEngineThreadCtx *det_ctx,
                             Packet *p, const Signature *s, const SigMatchCtx *ctx)
 {
     const DetectGeoipData *geoipdata = (const DetectGeoipData *)ctx;
@@ -462,10 +443,8 @@ static void DetectGeoipDataFree(void *ptr)
 {
     if (ptr != NULL) {
         DetectGeoipData *geoipdata = (DetectGeoipData *)ptr;
-#ifndef HAVE_LIBGEOIP
         if (geoipdata->mmdb_status == MMDB_SUCCESS)
             MMDB_close(&geoipdata->mmdb);
-#endif
         SCFree(geoipdata);
     }
 }
