@@ -36,7 +36,6 @@ enum RunModes {
     RUNMODE_DAG,
     RUNMODE_AFP_DEV,
     RUNMODE_NETMAP,
-    RUNMODE_TILERA_MPIPE,
     RUNMODE_UNITTEST,
     RUNMODE_NAPATECH,
     RUNMODE_UNIX_SOCKET,
@@ -79,7 +78,7 @@ const char *RunModeGetMainMode(void);
 void RunModeListRunmodes(void);
 void RunModeDispatch(int, const char *);
 void RunModeRegisterRunModes(void);
-void RunModeRegisterNewRunMode(int, const char *, const char *,
+void RunModeRegisterNewRunMode(enum RunModes, const char *, const char *,
                                int (*RunModeFunc)(void));
 void RunModeInitialize(void);
 void RunModeInitializeOutputs(void);
@@ -90,7 +89,8 @@ int RunModeOutputFileEnabled(void);
 /* bool indicating if filedata logger is enabled */
 int RunModeOutputFiledataEnabled(void);
 /** bool indicating if run mode is offline */
-bool IsRunModeOffline(int run_mode_to_check);
+bool IsRunModeOffline(enum RunModes run_mode_to_check);
+bool IsRunModeSystem(enum RunModes run_mode_to_check);
 
 void RunModeEnablesBypassManager(void);
 int RunModeNeedsBypassManager(void);
@@ -98,7 +98,6 @@ int RunModeNeedsBypassManager(void);
 #include "runmode-pcap.h"
 #include "runmode-pcap-file.h"
 #include "runmode-pfring.h"
-#include "runmode-tile.h"
 #include "runmode-nfq.h"
 #include "runmode-ipfw.h"
 #include "runmode-erf-file.h"
@@ -110,7 +109,7 @@ int RunModeNeedsBypassManager(void);
 #include "runmode-netmap.h"
 #include "runmode-windivert.h"
 
-int threading_set_cpu_affinity;
+extern int threading_set_cpu_affinity;
 extern float threading_detect_ratio;
 
 extern int debuglog_enabled;
